@@ -1,7 +1,7 @@
 /*
- * Miscellaneous algebraic manipulator routines.
+ * Miscellaneous routines for Mathomatic.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -83,6 +83,10 @@ init_gvars()
 	zero_token.level = 1;
 	zero_token.kind = CONSTANT;
 	zero_token.token.constant = 0.0;
+
+	one_token.level = 1;
+	one_token.kind = CONSTANT;
+	one_token.token.constant = 1.0;
 }
 
 /*
@@ -92,12 +96,12 @@ init_gvars()
 clean_up()
 {
 	init_gvars();
-#if	!SECURE
 	if (gfp != stdout) {
+#if	!SECURE
 		fclose(gfp);
+#endif
 		gfp = stdout;
 	}
-#endif
 }
 
 /*
@@ -274,9 +278,9 @@ long	v;
  * Substitute every instance of "v" in "equation" with "expression".
  */
 subst_var_with_exp(equation, np, expression, len, v)
-token_type	*equation;
-int		*np;		/* pointer to equation length */
-token_type	*expression;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to equation side length */
+token_type	*expression;	/* expression pointer */
 int		len;		/* expression length */
 long		v;
 {
@@ -304,7 +308,7 @@ long		v;
 int
 min_level(equation, n)
 token_type	*equation;	/* expression pointer */
-int		n;		/* expression size */
+int		n;		/* expression length */
 {
 	int		min1;
 	token_type	*p1, *ep;
@@ -347,9 +351,9 @@ char	*cp;
 {
 	int	i;
 
-	if (*cp == '\0')
+	if (*cp == '\0') {
 		i = cur_equation;
-	else {
+	} else {
 		i = decstrtol(cp, &cp) - 1;
 		if (extra_garbage(cp))
 			return -1;
@@ -363,12 +367,12 @@ char	*cp;
 /*
  * Parse an expression with equation space pull if the line starts with "#".
  *
- * Return true if successful.
+ * Returns the new string position or NULL on error.
  */
-int
+char	*
 parse_expr(equation, np, cp)
-token_type	*equation;	/* where the parsed expression is stored */
-int		*np;		/* pointer to parsed expression size */
+token_type	*equation;	/* where the parsed expression is stored (equation side) */
+int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
 {
 	int	i;
@@ -377,14 +381,23 @@ char		*cp;		/* string to parse */
 	cp1 = skip_space(cp);
 	if (*cp1 == '#') {
 		cp1++;
-		i = decstrtol(cp1, &cp2) - 1;
+		switch (*cp1) {
+		case '+':
+		case '-':
+			i = decstrtol(cp1, &cp2);
+			i = cur_equation + i;
+			break;
+		default:
+			i = decstrtol(cp1, &cp2) - 1;
+			break;
+		}
 		if (cp1 == cp2 || *cp2) {
 			error(_("Error parsing equation space number."));
-			return false;
+			return NULL;
 		}
 		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
 			error(_("Equation not defined."));
-			return false;
+			return NULL;
 		}
 		if (n_rhs[i]) {
 			blt(equation, rhs[i], n_rhs[i] * sizeof(token_type));
@@ -393,12 +406,9 @@ char		*cp;		/* string to parse */
 			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
 			*np = n_lhs[i];
 		}
-		return true;
+		return cp2;
 	}
-	cp = parse_section(equation, np, cp);
-	if (cp)
-		return true;
-	return false;
+	return parse_section(equation, np, cp);
 }
 
 /*
@@ -409,21 +419,23 @@ char		*cp;		/* string to parse */
  */
 int
 get_expr(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* where the parsed expression is stored (equation side) */
+int		*np;		/* pointer to the returned parsed expression length */
 {
-	char	buf[10000];
-	char	*cp1;
+	char	buf[DEFAULT_N_TOKENS];
+	char	*cp;
 
 	for (;;) {
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
 		if (!case_sensitive_flag) {
-			str_tolower(cp1);
+			str_tolower(cp);
 		}
-		if (parse_expr(equation, np, cp1))
+		cp = parse_expr(equation, np, cp);
+		if (cp && !extra_garbage(cp)) {
 			break;
+		}
 	}
 	return(*np > 0);
 }
@@ -435,24 +447,25 @@ int		*np;
  */
 int
 prompt_var(vp)
-long	*vp;
+long	*vp;	/* pointer to the returned variable */
 {
 	char	buf[MAX_CMD_LEN];
-	char	*cp1;
+	char	*cp;
 
-prompt_again:
-	my_strlcpy(prompt_str, _("Enter variable: "), sizeof(prompt_str));
-	if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
-		return false;
+	for (;;) {
+		my_strlcpy(prompt_str, _("Enter variable: "), sizeof(prompt_str));
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
+			return false;
+		}
+		if (*cp == '\0') {
+			return false;
+		}
+		cp = parse_var2(vp, cp);
+		if (cp == NULL || extra_garbage(cp)) {
+			continue;
+		}
+		return true;
 	}
-	cp1 = parse_var2(vp, cp1);
-	if (cp1 == NULL)
-		return false;
-	if (*cp1) {
-		error(_("Only one variable may be specified."));
-		goto prompt_again;
-	}
-	return true;
 }
 
 /*
@@ -462,7 +475,11 @@ int
 not_defined(i)
 int	i;	/* equation space number */
 {
-	if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
+	if (i < 0 || i >= n_equations) {
+		error(_("Invalid equation number."));
+		return true;
+	}
+	if (n_lhs[i] <= 0) {
 		error(_("Equation space empty."));
 		return true;
 	}
@@ -494,20 +511,23 @@ current_not_defined()
  * Returns "string" if successful or NULL on error.
  */
 char	*
-getstring(string, n)
+get_string(string, n)
 char	*string;	/* storage for input string */
 int	n;		/* maximum size of "string" in bytes */
 {
 #if	LIBRARY
+	error(_("Missing command line argument."));
 	return NULL;
 #else
+#if	READLINE
+	char	*cp;
+#endif
+
 	if (quiet_mode) {
 		prompt_str[0] = '\0';	/* don't display a prompt */
 	}
 	input_column = strlen(prompt_str);
 #if	READLINE
-	char	*cp;
-
 	if (!html_flag) {
 		cp = readline(prompt_str);
 		if (cp == NULL)
@@ -541,15 +561,15 @@ int	n;		/* maximum size of "string" in bytes */
 int
 get_yes_no()
 {
-	char	*cp1;
+	char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	for (;;) {
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		str_tolower(cp1);
-		switch (*cp1) {
+		str_tolower(cp);
+		switch (*cp) {
 		case 'n':
 			return false;
 		case 'y':
@@ -562,10 +582,13 @@ get_yes_no()
  * Store or display the result of a command.
  */
 return_result(en)
-int	en;
+int	en;	/* equation space number */
 {
 #if	LIBRARY
 	if (gfp == stdout) {
+		if (factor_int_flag) {
+			factor_int_sub(en);
+		}
 		result_str = list_equation(en, false);
 		return;
 	}
@@ -584,7 +607,8 @@ char	*cp;
 }
 
 /*
- * Process an equation number range given on the command line.
+ * Process an equation number range given in text string *cpp.
+ * Skip past all spaces and update *cpp to point to the next argument.
  *
  * Return true if successful,
  * with the starting equation number in "*ip"
@@ -599,7 +623,7 @@ int	*ip, *jp;
 
 	*cpp = skip_space(*cpp);
 #if	LIBRARY		/* only allow a single equation number when using as a library */
-	if (isascii(**cpp) && isdigit(**cpp)) {
+	if (isdigit(**cpp)) {
 		i = decstrtol(*cpp, cpp) - 1;
 	} else {
 		i = cur_equation;
@@ -616,8 +640,8 @@ int	*ip, *jp;
 		*ip = 0;
 		*jp = n_equations - 1;
 	} else {
-		if (isascii(**cpp) && isdigit(**cpp)) {
-			*ip = decstrtol(*cpp, cpp) - 1;
+		if (isdigit(**cpp)) {
+			*ip = strtol(*cpp, cpp, 10) - 1;
 		} else {
 			*ip = cur_equation;
 		}
@@ -630,12 +654,12 @@ int	*ip, *jp;
 				return false;
 			}
 			*jp = *ip;
+			*cpp = skip_space(*cpp);
 			return true;
 		}
 		(*cpp)++;
-		*cpp = skip_space(*cpp);
-		if (isascii(**cpp) && isdigit(**cpp)) {
-			*jp = decstrtol(*cpp, cpp) - 1;
+		if (isdigit(**cpp)) {
+			*jp = strtol(*cpp, cpp, 10) - 1;
 		} else {
 			*jp = cur_equation;
 		}
@@ -649,6 +673,7 @@ int	*ip, *jp;
 			*jp = i;
 		}
 	}
+	*cpp = skip_space(*cpp);
 	for (i = *ip; i <= *jp; i++) {
 		if (n_lhs[i] > 0) {
 			return true;
@@ -669,12 +694,14 @@ int	*ip, *jp;
  */
 int
 extra_garbage(cp)
-char	*cp;
+char	*cp;	/* command line */
 {
-	cp = skip_space(cp);
-	if (*cp) {
-		error(_("Extra characters on command line."));
-		return true;
+	if (cp) {
+		cp = skip_space(cp);
+		if (*cp) {
+			error(_("Extra characters encountered."));
+			return true;
+		}
 	}
 	return false;
 }
@@ -703,7 +730,7 @@ char	*
 skip_space(cp)
 char	*cp;	/* character pointer */
 {
-	while (*cp && isascii(*cp) && isspace(*cp))
+	while (*cp && isspace(*cp))
 		cp++;
 	return cp;
 }
@@ -727,6 +754,7 @@ char	*cp, **cpp;
 
 /*
  * Skip over a parameter (command line argument) in a character string.
+ * Parameters are separated with spaces.
  *
  * Returns a string pointer to the next parameter.
  */
@@ -734,7 +762,7 @@ char	*
 skip_param(cp)
 char	*cp;	/* character pointer */
 {
-	while (*cp && (!isascii(*cp) || (!isspace(*cp) && *cp != '='))) {
+	while (*cp && !isspace(*cp) && *cp != '=') {
 		cp++;
 	}
 	cp = skip_space(cp);
@@ -756,9 +784,9 @@ char	*cp1, *cp2;
 {
 	char	*cp1a, *cp2a;
 
-	for (cp1a = cp1; *cp1a && (!isascii(*cp1a) || !isspace(*cp1a)); cp1a++)
+	for (cp1a = cp1; *cp1a && !isspace(*cp1a); cp1a++)
 		;
-	for (cp2a = cp2; *cp2a && (!isascii(*cp2a) || !isspace(*cp2a)); cp2a++)
+	for (cp2a = cp2; *cp2a && !isspace(*cp2a); cp2a++)
 		;
 	return strncasecmp(cp1, cp2, max(cp1a - cp1, cp2a - cp2));
 }
@@ -769,7 +797,7 @@ char	*cp1, *cp2;
 int
 level_plus(p1, n1, level)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 int		level;	/* parentheses level number */
 {
 	int	i;
@@ -794,7 +822,7 @@ int		level;	/* parentheses level number */
 int
 level1_plus(p1, n1)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 {
 	int	level;
 
@@ -808,7 +836,7 @@ int		n1;	/* expression size */
 int
 var_count(p1, n1)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 {
 	int	i;
 	int	count;
@@ -828,7 +856,7 @@ int		n1;	/* expression size */
 int
 exp_contains_infinity(p1, n1)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 {
 	int	i;
 
