@@ -1,21 +1,21 @@
 /*
- * Algebraic manipulator commands.
+ * C code for the Mathomatic commands.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
 
-#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in "optimize" command. */
+#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in optimize command. */
 
 static int	sum_product();
 static int	find_more();
 static int	opt_es();
 static int	complex_func();
-static int	compare_rhs();
 static int	elim_sub();
 static int	edit_sub();
 
+/* Global variables for the optimize command. */
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
 
@@ -25,10 +25,12 @@ static int	last_temp_var = 0;
  */
 int
 version_cmd(cp)
-char	*cp;
+char	*cp;	/* the command line argument */
 {
-	if (extra_garbage(cp))
+	if (extra_garbage(cp)) {
+		/* if anything on the command line, fail */
 		return false;
+	}
 	return version_report();
 }
 
@@ -51,9 +53,6 @@ version_report()
 #if	READLINE
 	fprintf(gfp, "READLINE ");
 #endif
-#if	BASICS
-	fprintf(gfp, "BASICS ");
-#endif
 #if	SILENT
 	fprintf(gfp, "SILENT ");
 #endif
@@ -71,6 +70,42 @@ version_report()
 }
 #endif
 
+/*
+ * The solve command.
+ */
+int
+solve_cmd(cp)
+char	*cp;
+{
+	int	i;
+	char	buf[MAX_CMD_LEN];
+
+	if (current_not_defined()) {
+		return false;
+	}
+	if (*cp == '\0') {
+		my_strlcpy(prompt_str, _("Enter variable: "), sizeof(prompt_str));
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
+			return false;
+		}
+		if (!case_sensitive_flag) {
+			str_tolower(cp);
+		}
+	}
+	i = next_espace();
+	if ((cp = parse_equation(i, cp)) != NULL) {
+		if (n_lhs[i] && n_rhs[i]) {
+			error(_("Invalid argument."));
+		} else if (!extra_garbage(cp) && solve(i, cur_equation)) {
+			return_result(cur_equation);
+			return true;
+		}
+	}
+	n_lhs[i] = 0;
+	n_rhs[i] = 0;
+	return false;
+}
+
 /*
  * The sum command.
  */
@@ -101,13 +136,12 @@ int	product_flag;	/* true for product, otherwise sum */
 {
 	int		i, j;
 	long		v = 0;
-	char		*cp1;
 	double		start, end, step = 1.0;
 	int		result_en;
 	int		n, ns;
 	token_type	*dest, *source;
 	int		count_down;		/* if true, count down, otherwise count up */
-	char		buf[MAX_CMD_LEN];
+	char		*cp1, buf[MAX_CMD_LEN];
 
 	if (current_not_defined()) {
 		return false;
@@ -156,7 +190,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	} else {
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), "%s = ", var_str);
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp1 = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	start = strtod(cp1, &cp);
@@ -169,7 +203,7 @@ int	product_flag;	/* true for product, otherwise sum */
 		cp1 = cp;
 	} else {
 		my_strlcpy(prompt_str, _("To: "), sizeof(prompt_str));
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp1 = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	end = strtod(cp1, &cp);
@@ -192,12 +226,10 @@ int	product_flag;	/* true for product, otherwise sum */
 	if (fmod(fabs(start - end) / step, 1.0)) {
 		printf(_("Warning: end value not reached.\n"));
 	}
-	dest[0].kind = CONSTANT;
-	dest[0].level = 1;
 	if (product_flag) {
-		dest[0].token.constant = 1.0;
+		dest[0] = one_token;
 	} else {
-		dest[0].token.constant = 0.0;
+		dest[0] = zero_token;
 	}
 	n = 1;
 	for (; count_down ? (start >= end) : (start <= end); count_down ? (start -= step) : (start += step)) {
@@ -241,59 +273,52 @@ int	product_flag;	/* true for product, otherwise sum */
 	return true;
 }
 
-#if	!BASICS && !LIBRARY
+#if	!LIBRARY
 /*
  * This function is for the "optimize" command.
- * It finds and substitutes all occurrences of "en"
- * in "equation".
+ * It finds and substitutes all occurrences of the RHS of "en" in "equation".
  * It should be called repeatedly until it returns false.
  */
 static int
 find_more(equation, np, en)
-token_type	*equation;
-int		*np;		/* length of "equation" */
-int		en;		/* equation number */
+token_type	*equation;	/* expression pointer */
+int		*np;		/* pointer to length of expression */
+int		en;		/* equation space number */
 {
 	int	i, j, k;
 	int	level;
 	int	diff_sign;
-	int	found_se;
-	long	v;
+	int	found_se = true;
 
-	found_se = true;
 	for (level = 1; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
-			for (j = i; j < *np && equation[j].level > level; j += 2) {
-			}
+			for (j = i; j < *np && equation[j].level > level; j += 2)
+				;
 			if (j == i) {
 				continue;
 			}
 			found_se = true;
 			k = i - 1;
-			if ((j - k) >= OPT_MIN_SIZE
-			    && se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
-				v = lhs[en][0].token.variable;
+			if ((j - k) >= OPT_MIN_SIZE && se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
 				if (diff_sign) {
 					blt(&equation[i+2], &equation[j], (*np - j) * sizeof(token_type));
 					*np -= (j - (i + 2));
-					equation[k].level = level + 1;
+					level++;
+					equation[k].level = level;
 					equation[k].kind = CONSTANT;
 					equation[k].token.constant = -1.0;
 					k++;
-					equation[k].level = level + 1;
+					equation[k].level = level;
 					equation[k].kind = OPERATOR;
 					equation[k].token.operatr = TIMES;
 					k++;
-					equation[k].level = level + 1;
-					equation[k].kind = VARIABLE;
-					equation[k].token.variable = v;
 				} else {
 					blt(&equation[i], &equation[j], (*np - j) * sizeof(token_type));
 					*np -= (j - i);
-					equation[k].level = level;
-					equation[k].kind = VARIABLE;
-					equation[k].token.variable = v;
 				}
+				equation[k].level = level;
+				equation[k].kind = VARIABLE;
+				equation[k].token.variable = lhs[en][0].token.variable;
 				return true;
 			}
 		}
@@ -303,8 +328,7 @@ int		en;		/* equation number */
 
 /*
  * This function is for the "optimize" command.
- * It finds and replaces all repeated expressions in
- * "equation" with temporary variables.
+ * It finds and replaces all repeated expressions in "equation" with temporary variables.
  * It also creates a new equation for each temporary variable.
  * It should be called repeatedly until it returns false.
  */
@@ -313,8 +337,7 @@ opt_es(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	i, j, k;
-	int	i1, i2, jj1, k1;
+	int	i, j, k, i1, i2, jj1, k1;
 	int	level, level1;
 	int	diff_sign;
 	int	found_se, found_se1;
@@ -323,8 +346,8 @@ int		*np;
 	found_se = true;
 	for (level = 1; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
-			for (j = i; j < *np && equation[j].level > level; j += 2) {
-			}
+			for (j = i; j < *np && equation[j].level > level; j += 2)
+				;
 			if (j == i) {
 				continue;
 			}
@@ -362,24 +385,22 @@ int		*np;
 						if (diff_sign) {
 							blt(&equation[i1+2], &equation[jj1], (*np - jj1) * sizeof(token_type));
 							*np -= (jj1 - (i1 + 2));
-							equation[k1].level = level1 + 1;
+							level1++;
+							equation[k1].level = level1;
 							equation[k1].kind = CONSTANT;
 							equation[k1].token.constant = -1.0;
 							k1++;
-							equation[k1].level = level1 + 1;
+							equation[k1].level = level1;
 							equation[k1].kind = OPERATOR;
 							equation[k1].token.operatr = TIMES;
 							k1++;
-							equation[k1].level = level1 + 1;
-							equation[k1].kind = VARIABLE;
-							equation[k1].token.variable = v;
 						} else {
 							blt(&equation[i1], &equation[jj1], (*np - jj1) * sizeof(token_type));
 							*np -= (jj1 - i1);
-							equation[k1].level = level1;
-							equation[k1].kind = VARIABLE;
-							equation[k1].token.variable = v;
 						}
+						equation[k1].level = level1;
+						equation[k1].kind = VARIABLE;
+						equation[k1].token.variable = v;
 						blt(&equation[i], &equation[j], (*np - j) * sizeof(token_type));
 						*np -= j - i;
 						equation[k].level = level;
@@ -409,9 +430,8 @@ int
 optimize_cmd(cp)
 char	*cp;
 {
-	int	i, j, k;
+	int	i, j, k, i1;
 	int	start, stop;
-	int	i1;
 	int	rv = false;
 	int	flag;
 
@@ -465,12 +485,15 @@ int
 push_cmd(cp)
 char	*cp;
 {
-	int	i;
+	int	i, j;
 
-	if ((i = get_default_en(cp)) < 0)
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
-	push_en(i);
-	debug_string(0, _("Equation pushed.  Hit the UP key to access."));
+	}
+	for (; i <= j; i++) {
+		push_en(i);
+	}
+	debug_string(0, _("Expression(s) pushed.  Press the UP key to access."));
 	return true;
 }
 
@@ -497,7 +520,11 @@ char	*cp;
 
 	if (*cp == '\0') {
 		fprintf(gfp, _("Options are set as follows:\n\n"));
+
 		output_options();
+
+		fprintf(gfp, "columns = %d\n", screen_columns);
+
 #if	!SECURE
 		if (getcwd(buf, sizeof(buf))) {
 			fprintf(gfp, "directory = %s\n", buf);
@@ -509,13 +536,14 @@ char	*cp;
 }
 
 /*
- * Output the set options in a format suitable for RC_FILE.
+ * Output the current set options in a format suitable for RC_FILE.
  */
 output_options()
 {
 #if	!SILENT
 	fprintf(gfp, "debug_level = %d\n", debug_level);
 #endif
+
 	if (!case_sensitive_flag) {
 		fprintf(gfp, "no ");
 	}
@@ -531,8 +559,6 @@ output_options()
 	}
 	fprintf(gfp, "bold_colors\n");
 
-	fprintf(gfp, "columns = %d\n", screen_columns);
-
 	if (!groupall) {
 		fprintf(gfp, "no ");
 	}
@@ -552,13 +578,18 @@ output_options()
 		fprintf(gfp, "no ");
 	}
 	fprintf(gfp, "finance\n");
+
+	if (!factor_int_flag) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "factor_integers\n");
 }
 
 /*
  * Return true if "cp" points to a negative word.
  */
 int
-isno(cp)
+is_no(cp)
 char	*cp;
 {
 	if (strcmp_tospace(cp, "no") == 0
@@ -570,7 +601,7 @@ char	*cp;
 }
 
 /*
- * Handle parsing of options for "set" command.
+ * Handle parsing of options for the set command.
  *
  * Return false if error.
  */
@@ -580,11 +611,12 @@ char	*cp;
 {
 	int	negate;
 
+next_option:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-	negate = isno(cp);
+	negate = is_no(cp);
 	if (negate) {
 		cp = skip_param(cp);
 	}
@@ -601,7 +633,6 @@ char	*cp;
 #if	!SILENT
 	if (strncasecmp(cp, "debug", 5) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		if (negate) {
 			debug_level = 0;
 		} else {
@@ -609,14 +640,13 @@ char	*cp;
 				error(_("Please specify a debug level number."));
 				return false;
 			}
-			debug_level = atoi(cp);
+			debug_level = decstrtol(cp, &cp);
 		}
-		return true;
+		goto next_option;
 	}
 #endif
 	if (strncasecmp(cp, "columns", 7) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		if (negate) {
 			screen_columns = 0;
 		} else {
@@ -624,53 +654,51 @@ char	*cp;
 				error(_("Please specify how wide the screen is in columns."));
 				return false;
 			}
-			screen_columns = atoi(cp);
+			screen_columns = decstrtol(cp, &cp);
 		}
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		case_sensitive_flag = !negate;
-		return true;
+		goto next_option;
 	}
-	if (strncasecmp(cp, "display2d", 7) == 0) {
+	if (strncasecmp(cp, "display2d", 9) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		groupall = !negate;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		preserve_roots = !negate;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "true", 4) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		true_modulus = !negate;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "color", 5) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		color_flag = !negate;
 		cur_color = -1;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "bold", 4) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		bold_colors = !negate;
 		cur_color = -1;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "finance", 7) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		finance_option = !negate;
-		return true;
+		goto next_option;
+	}
+	if (strncasecmp(cp, "factor", 6) == 0) {
+		cp = skip_param(cp);
+		factor_int_flag = !negate;
+		goto next_option;
 	}
 	error(_("Unknown set option."));
 	return false;
@@ -690,12 +718,12 @@ char	*cp;
 		cp = _("Please Press Enter");
 	}
 	snprintf(prompt_str, sizeof(prompt_str), " ==== %s ==== ", cp);
-	if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+	if ((cp1 = get_string(buf, sizeof(buf))) == NULL) {
 		return false;
 	}
-	if (strncasecmp(cp1, "quit", 4) == 0
-	    || strncasecmp(cp1, "abort", 5) == 0)
+	if (strncasecmp(cp1, "quit", 4) == 0) {
 		return false;
+	}
 	return true;
 }
 
@@ -748,9 +776,12 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	int		has_imag, has_real;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
+	long		v;
 
-	if ((i = get_default_en(cp)) < 0)
+	if (current_not_defined()) {
 		return false;
+	}
+	i = cur_equation;
 	j = next_espace();
 	if (n_rhs[i]) {
 		source = rhs[i];
@@ -763,9 +794,18 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 		dest = lhs[j];
 		np = &n_lhs[j];
 	}
+	v = IMAGINARY;
+	if (*cp) {
+		cp = parse_var2(&v, cp);
+		if (cp == NULL) {
+			return false;
+		}
+		if (extra_garbage(cp))
+			return false;
+	}
 	simp_loop(source, nps);
 	uf_simp(source, nps);
-	factorv(source, nps, IMAGINARY);
+	factorv(source, nps, v);
 	partial_flag = false;
 	uf_simp(source, nps);
 	partial_flag = true;
@@ -783,7 +823,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 			    && (source[k].token.operatr == PLUS || source[k].token.operatr == MINUS)) {
 				break;
 			}
-			if (source[k].kind == VARIABLE && source[k].token.variable == IMAGINARY) {
+			if (source[k].kind == VARIABLE && source[k].token.variable == v) {
 				found_imag = true;
 			}
 		}
@@ -799,12 +839,8 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 			n1 += (k - beg);
 		}
 	}
-	if (!has_imag) {
-		error(_("No imaginary parts found.  Expression is not complex."));
-		return false;
-	}
-	if (!has_real) {
-		error(_("No real parts found.  Expression is not complex."));
+	if (!has_imag || !has_real) {
+		error(_("Failed, expression is not a mix."));
 		return false;
 	}
 	simp_divide(dest, &n1);
@@ -916,20 +952,19 @@ calculate_cmd(cp)
 char	*cp;
 {
 	int		i, j, k;
-	long		v, last_v, it_v;
+	long		v, last_v, it_v = 0;
 	long		counter;
 	long		counter_max;
 	sign_array_type	sa_mark;
 	sign_array_type	sa_value;
-	int		iterations;
+	int		iterations = 1;
 	int		n;
 	token_type	*source;
 
-	iterations = 1;
-	it_v = 0;
-	i = cur_equation;
-	if (not_defined(i))
+	if (current_not_defined()) {
 		return false;
+	}
+	i = cur_equation;
 	if (n_rhs[i]) {
 		source = rhs[i];
 		n = n_rhs[i];
@@ -1006,8 +1041,7 @@ char	*cp;
 	for (j = 0; j < ARR_CNT(sa_mark); j++)
 		sa_mark[j] = false;
 	for (j = 0; j < n_trhs; j += 2) {
-		if (trhs[j].kind == VARIABLE
-		    && (trhs[j].token.variable & VAR_MASK) == SIGN) {
+		if (trhs[j].kind == VARIABLE && (trhs[j].token.variable & VAR_MASK) == SIGN) {
 			sa_mark[(trhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
 		}
 	}
@@ -1028,8 +1062,7 @@ char	*cp;
 			}
 		}
 		for (j = 0; j < n_tlhs; j += 2) {
-			if (tlhs[j].kind == VARIABLE
-			    && (tlhs[j].token.variable & VAR_MASK) == SIGN) {
+			if (tlhs[j].kind == VARIABLE && (tlhs[j].token.variable & VAR_MASK) == SIGN) {
 				if (sa_value[(tlhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
 					tlhs[j].kind = CONSTANT;
 					tlhs[j].token.constant = -1.0;
@@ -1059,13 +1092,16 @@ char	*cp;
 		if (k)
 			fprintf(gfp, ":\n");
 		calc_simp(tlhs, &n_tlhs);
+		if (factor_int_flag) {
+			factor_int(tlhs, &n_tlhs);
+		}
 		fprintf(gfp, " ");
 		if (n_rhs[i]) {
 			list_proc(lhs[i], n_lhs[i], false);
 			fprintf(gfp, " = ");
 		}
 		list_proc(tlhs, n_tlhs, false);
-		fprintf(gfp, "\n");
+		fprintf(gfp, "\n\n");
 	}
 	return true;
 }
@@ -1079,18 +1115,27 @@ clear_cmd(cp)
 char	*cp;
 {
 	int	i, j;
+	char	*cp1;
 
-	if (is_all(cp)) {
-		clear_all();
-	} else {
-		if (!get_range_eol(&cp, &i, &j)) {
-			return false;
+	do {
+		cp1 = cp;
+		if (is_all(cp)) {
+			clear_all();
+			return true;
+		} else {
+			if (!get_range(&cp, &i, &j)) {
+				return false;
+			}
+			if (*cp && cp == cp1) {
+				error(_("Invalid argument."));
+				return false;
+			}
+			for (; i <= j; i++) {
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+			}
 		}
-		for (; i <= j; i++) {
-			n_lhs[i] = 0;
-			n_rhs[i] = 0;
-		}
-	}
+	} while (*cp);
 	return true;
 }
 
@@ -1167,8 +1212,8 @@ char	*cp;
 				return true;
 			}
 			debug_string(0, _("Simplifying both expressions..."));
-			simpa_side(lhs[i], &n_lhs[i], false, true);
-			simpa_side(lhs[j], &n_lhs[j], false, true);
+			simpa_side(lhs[i], &n_lhs[i], true);
+			simpa_side(lhs[j], &n_lhs[j], true);
 #if	!SILENT
 			if (debug_level >= 0) {
 				list_sub(i);
@@ -1199,8 +1244,8 @@ char	*cp;
 			goto times_neg1;
 		}
 		debug_string(0, _("Simplifying both equations..."));
-		simpa_side(rhs[i], &n_rhs[i], false, true);
-		simpa_side(rhs[j], &n_rhs[j], false, true);
+		simpa_side(rhs[i], &n_rhs[i], true);
+		simpa_side(rhs[j], &n_rhs[j], true);
 #if	!SILENT
 		if (debug_level >= 0) {
 			list_sub(i);
@@ -1229,8 +1274,8 @@ char	*cp;
 		return true;
 	}
 	debug_string(0, _("Simplifying both equations..."));
-	simpa_side(rhs[i], &n_rhs[i], false, true);
-	simpa_side(rhs[j], &n_rhs[j], false, true);
+	simpa_side(rhs[i], &n_rhs[i], true);
+	simpa_side(rhs[j], &n_rhs[j], true);
 	if (compare_rhs(i, j, &diff_sign)) {
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
@@ -1275,8 +1320,7 @@ char	*cp;
 	long		v, v_tmp;
 	int		i, j;
 	int		nl, nr;
-	double		d1, d2;
-	double		d3, d4;
+	double		d1, d2, d3, d4;
 	complexs	c1, c2, c3;
 
 	v = 0;
@@ -1363,14 +1407,11 @@ int
 eliminate_cmd(cp)
 char	*cp;
 {
-	long	v, last_v, v1;
-	int	i;
-	int	n;
-	int	using_flag;
+	long	v, last_v, v1, va[MAX_VARS];
+	int	i, n;
+	int	using_flag, did_something = false;
 	char	used[N_EQUATIONS];
-	int	did_something = false;
 	int	vc = 0;
-	long	va[MAX_VARS];
 
 	for (i = 0; i < ARR_CNT(used); i++)
 		used[i] = false;
@@ -1422,12 +1463,16 @@ next_var:
 	} else {
 		if (did_something) {
 			return_result(cur_equation);
+		} else {
+			error(_("No substitutions made."));
 		}
 		return did_something;
 	}
 	if (!var_in_equation(cur_equation, v)) {
+#if	!SILENT
 		list_var(v, 0);
 		printf(_("Variable (%s) not found in current equation.\n"), var_str);
+#endif
 		goto next_var;
 	}
 	using_flag = (strcmp_tospace(cp, "using") == 0);
@@ -1448,8 +1493,6 @@ next_var:
 		i = cur_equation;
 		for (;; n++) {
 			if (n >= n_equations) {
-				list_var(v, 0);
-				printf(_("Variable (%s) not found in any other equation.\n"), var_str);
 				goto next_var;
 			}
 			if (i <= 0)
@@ -1495,7 +1538,6 @@ long	v;
 	return true;
 }
 
-#if	!LIBRARY
 /*
  * The flist command.
  *
@@ -1508,7 +1550,8 @@ char	*cp;
 	int	i, j;
 	int	factor_flag;
 
-	if (factor_flag = (strcmp_tospace(cp, "factor") == 0)) {
+	factor_flag = (strcmp_tospace(cp, "factor") == 0);
+	if (factor_flag) {
 		cp = skip_param(cp);
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -1517,16 +1560,20 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			group_sub(i);
-			if (factor_flag) {
-				factor_int(lhs[i], &n_lhs[i]);
-				factor_int(rhs[i], &n_rhs[i]);
+			if (factor_flag || factor_int_flag) {
+				factor_int_sub(i);
 			}
+#if	LIBRARY
+			if (gfp == stdout) {
+				result_str = list_equation(i, false);
+				return true;
+			}
+#endif
 			flist_sub(i);
 		}
 	}
 	return true;
 }
-#endif
 
 /*
  * The list command.
@@ -1549,10 +1596,13 @@ char	*cp;
 		return false;
 	}
 	for (; i <= j; i++) {
+		if (factor_int_flag) {
+			factor_int_sub(i);
+		}
 #if	LIBRARY
 		if (gfp == stdout) {
 			result_str = list_equation(i, export_flag);
-			continue;
+			return true;
 		}
 #endif
 		list1_sub(i, export_flag);
@@ -1560,7 +1610,7 @@ char	*cp;
 	return true;
 }
 
-#if	!BASICS && !LIBRARY
+#if	!LIBRARY
 /*
  * The code command.
  */
@@ -1569,17 +1619,17 @@ code_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	java_flag = 0;
+	int	language = 0;
 	int	int_flag = false;
 
 	if (strcmp_tospace(cp, "c") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
-		java_flag = 1;
+		language = 1;
 	} else if (strcmp_tospace(cp, "python") == 0) {
 		cp = skip_param(cp);
-		java_flag = 2;
+		language = 2;
 	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
 		cp = skip_param(cp);
 		int_flag = true;
@@ -1590,7 +1640,9 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
-				printf(_("Equation #%d is not an equation.\n"), i + 1);
+#if	!SILENT
+				printf(_("#%d is not a solved equation.\n"), i + 1);
+#endif
 				continue;
 			}
 			in_calc_cmd = true;
@@ -1602,11 +1654,13 @@ char	*cp;
 			in_calc_cmd = false;
 			if (int_flag) {
 				if (!int_expr(rhs[i], n_rhs[i])) {
-					printf(_("Equation #%d is not an integer equation.\n"), i + 1);
+#if	!SILENT
+					printf(_("#%d is not an integer equation.\n"), i + 1);
+#endif
 					continue;
 				}
 			}
-			list_c_equation(i, java_flag, int_flag);
+			list_c_equation(i, language, int_flag);
 		}
 	}
 	return true;
@@ -1630,10 +1684,10 @@ char	*cp;
 	char	*cp1;
 
 	cp_start = cp;
-	i = cur_equation;
-	if (not_defined(i)) {
+	if (current_not_defined()) {
 		return false;
 	}
+	i = cur_equation;
 	if (strcmp_tospace(cp, "constants") == 0) {
 		cp = skip_param(cp);
 		if (extra_garbage(cp))
@@ -1648,6 +1702,7 @@ char	*cp;
 		if (strcmp_tospace(cp, "with") == 0) {
 			if (vc)
 				break;
+			error(_("No variables specified."));
 			return false;
 		}
 		if (vc >= ARR_CNT(va)) {
@@ -1705,8 +1760,7 @@ char	*cp;
 				if (!case_sensitive_flag) {
 					str_tolower(cp1);
 				}
-				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL
-				    || n <= 0) {
+				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL || n <= 0) {
 					return false;
 				}
 				if (extra_garbage(cp1))
@@ -1751,15 +1805,9 @@ simplify_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	poly_flag = false;
-	int	quick_flag = false;
-	int	symb = false;
+	int	quick_flag = false, symb = false;
 
 	for (;; cp = skip_param(cp)) {
-		if (strncasecmp(cp, "poly", 4) == 0) {
-			poly_flag = true;
-			continue;
-		}
 		if (strncasecmp(cp, "symbolic", 4) == 0) {
 			symb = true;
 			continue;
@@ -1776,8 +1824,8 @@ char	*cp;
 	symb_flag = symb;
 	for (; i <= j; i++) {
 		if (n_lhs[i]) {
-			simpa_side(lhs[i], &n_lhs[i], poly_flag, quick_flag);
-			simpa_side(rhs[i], &n_rhs[i], poly_flag, quick_flag);
+			simpa_side(lhs[i], &n_lhs[i], quick_flag);
+			simpa_side(rhs[i], &n_rhs[i], quick_flag);
 			return_result(i);
 		}
 	}
@@ -1802,7 +1850,7 @@ char	*cp;
 		cp = skip_param(cp);
 		if (*cp == '\0') {
 			my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
-			cp = getstring(buf, sizeof(buf));
+			cp = get_string(buf, sizeof(buf));
 			if (cp == NULL)
 				return false;
 		}
@@ -1850,7 +1898,8 @@ char	*cp;
 	int	i, j;
 	int	fully_flag;
 
-	if (fully_flag = (strncasecmp(cp, "fully", 4) == 0)) {
+	fully_flag = (strncasecmp(cp, "fully", 4) == 0);
+	if (fully_flag) {
 		cp = skip_param(cp);
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -1918,7 +1967,7 @@ char	*cp;
 		printf(_("Read operation aborted.\n"));
 		goto end_read;
 	}
-	while (cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) {
+	while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
 		default_color();
 		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
 		printf("%s", cp);
