@@ -1,7 +1,7 @@
 /*
- * Floating point complex number routines.
+ * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -32,12 +32,12 @@ char	*cp;
 	double		d, k;
 	double		root;
 	double		radius, theta;
-	double		radius_root;
+	double		radius_root = 0.0;
 	char		buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter root (positive integer): "), sizeof(prompt_str));
-		if ((cp = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	root = strtod(cp, &cp);
@@ -48,7 +48,7 @@ char	*cp;
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter real part (X): "), sizeof(prompt_str));
-		if ((cp = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	c.re = strtod(cp, &cp);
@@ -59,7 +59,7 @@ char	*cp;
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter imaginary part (Y): "), sizeof(prompt_str));
-		if ((cp = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	c.im = strtod(cp, &cp);
@@ -78,7 +78,7 @@ char	*cp;
 	}
 	check_err();
 	fprintf(gfp, _("\nThe polar coordinates are:\n%.12g amplitude and %.12g radians (%.12g degrees).\n\n"),
-	    radius, theta, theta * 180 / PI);
+	    radius, theta, theta * 180 / M_PI);
 	if (root) {
 		if (c.im == 0.0) {
 			fprintf(gfp, _("The %.12g roots of %.12g^(1/%.12g) are:\n\n"), root, c.re, root);
@@ -87,8 +87,8 @@ char	*cp;
 		}
 		for (k = 0.0; k < root; k += 1.0) {
 /* add constants to theta and convert back to rectangular coordinates */
-			c2.re = radius_root * cos((theta + 2.0 * k * PI) / root);
-			c2.im = radius_root * sin((theta + 2.0 * k * PI) / root);
+			c2.re = radius_root * cos((theta + 2.0 * k * M_PI) / root);
+			c2.im = radius_root * sin((theta + 2.0 * k * M_PI) / root);
 			complex_fixup(&c2);
 			if (c2.im == 0.0) {
 				fprintf(gfp, "%.12g\n", c2.re);
@@ -112,22 +112,23 @@ char	*cp;
 #endif
 
 /*
- * Approximate roots of complex numbers (complex^real
- * and real^complex and complex^complex).
- * Only gives one root, so this is not perfect.
+ * Approximate roots of complex numbers:
+ * (complex^real) and (real^complex) and (complex^complex).
  *
  * Returns true if expression was modified.
  */
 int
 complex_root_simp(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 {
 	int		i, j;
 	int		level;
 	int		len;
 	complexs	c, p;
+	int		modified = false;
 
+start_over:
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr != POWER)
 			continue;
@@ -145,7 +146,6 @@ int		*np;
 		if (c.im == 0.0 && p.im == 0.0)
 			continue;
 		i += len + 1;
-		debug_string(1, _("Warning: Complex number root taken."));
 		c = complex_pow(c, p);
 		if (*np + 5 - (i - j) > n_tokens) {
 			error_huge();
@@ -160,20 +160,22 @@ int		*np;
 		equation[j].kind = OPERATOR;
 		equation[j].token.operatr = PLUS;
 		j++;
-		equation[j].level = level + 1;
+		level++;
+		equation[j].level = level;
 		equation[j].kind = CONSTANT;
 		equation[j].token.constant = c.im;
 		j++;
-		equation[j].level = level + 1;
+		equation[j].level = level;
 		equation[j].kind = OPERATOR;
 		equation[j].token.operatr = TIMES;
 		j++;
-		equation[j].level = level + 1;
+		equation[j].level = level;
 		equation[j].kind = VARIABLE;
 		equation[j].token.variable = IMAGINARY;
-		return true;
+		modified = true;
+		goto start_over;
 	}
-	return false;
+	return modified;
 }
 
 /*
@@ -183,7 +185,7 @@ int		*np;
  */
 int
 get_constant(equation, n, dp)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 double		*dp;		/* pointer to returned double */
 {
@@ -208,7 +210,7 @@ double		*dp;		/* pointer to returned double */
  */
 int
 parse_complex(equation, n, cp)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 complexs	*cp;		/* pointer to returned complex number */
 {
@@ -237,26 +239,36 @@ complexs	*cp;		/* pointer to returned complex number */
 			imag_cnt++;
 			break;
 		case OPERATOR:
+			level2 = equation[j].level;
 			switch (equation[j].token.operatr) {
 			case TIMES:
 				if (++times_cnt > 1)
 					return false;
-				level2 = equation[j].level;
 				if (equation[j-1].level != level2 || equation[j+1].level != level2)
 					return false;
-				if (equation[j-1].kind == VARIABLE
-				    && equation[j-1].token.variable == IMAGINARY) {
+				if (equation[j-1].kind == VARIABLE && equation[j-1].token.variable == IMAGINARY) {
 					if (!get_constant(&equation[j+1], 1, &c.im))
 						return false;
 					continue;
 				}
-				if (equation[j+1].kind == VARIABLE
-				    && equation[j+1].token.variable == IMAGINARY) {
+				if (equation[j+1].kind == VARIABLE && equation[j+1].token.variable == IMAGINARY) {
 					if (!get_constant(&equation[j-1], 1, &c.im))
 						return false;
 					continue;
 				}
 				return false;
+			case DIVIDE:
+				if (++times_cnt > 1)
+					return false;
+				if (equation[j-1].level != level2 || equation[j+1].level != level2)
+					return false;
+				if (equation[j-1].kind == VARIABLE && equation[j-1].token.variable == IMAGINARY) {
+					if (!get_constant(&equation[j+1], 1, &c.im))
+						return false;
+					c.im = 1.0 / c.im;
+					continue;
+				}
+				return false;
 			case MINUS:
 				if (imag_cnt) {
 					c.im = -c.im;
@@ -264,7 +276,6 @@ complexs	*cp;		/* pointer to returned complex number */
 			case PLUS:
 				if (++plus_cnt > 1)
 					return false;
-				level2 = equation[j].level;
 				if (equation[j-1].level == level2 && get_constant(&equation[j-1], 1, &c.re)) {
 					continue;
 				}
