@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator externals.
+ * Mathomatic global variable definitions.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 extern int		n_tokens;
@@ -23,10 +23,12 @@ extern token_type	*trhs;
 extern token_type	*scratch;
 
 extern token_type	zero_token;
+extern token_type	one_token;
 
 extern char		*var_names[MAX_VAR_NAMES];
 
 extern int		case_sensitive_flag;
+extern int		factor_int_flag;
 extern int		groupall;
 extern int		in_calc_cmd;
 extern int		preserve_roots;
@@ -34,7 +36,6 @@ extern int		true_modulus;
 extern int		screen_columns;
 extern int		screen_rows;
 extern int		finance_option;
-extern int		starstar;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
