@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator symbolic factorizing routines.
+ * Mathomatic symbolic factorizing routines.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  *
  * There are proper mathematical names for many algebraic rules.
  * I obviously don't know what they are.
@@ -49,12 +49,10 @@ int		*np;
 }
 
 /*
- * Factor equation side.
- * (a*c + b*c) -> (c*(a + b)).
+ * Factor equation side: (a*c + b*c) -> (c*(a + b)).
  *
- * If "v" equals 0L, factor anything,
- * including identical bases raised to powers:
- * (x^2 + x) -> (x*(x + 1)).
+ * If "v" and "d" equals 0, factor anything,
+ * including identical bases raised to powers: (x^2 + x) -> (x*(x + 1)).
  * If "d" equals 1.0, only factor identical bases raised to the power of a constant.
  *
  * If "v" is a variable, only factor expressions containing that variable.
@@ -65,10 +63,10 @@ int		*np;
  */
 int
 factor_plus(equation, np, v, d)
-token_type	*equation;
-int		*np;
-long		v;
-double		d;
+token_type	*equation;	/* pointer to beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* variable */
+double		d;		/* control exponent */
 {
 	return fplus_recurse(equation, np, 0, 1, v, d, 0);
 }
@@ -177,17 +175,11 @@ double		d;
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
-	switch (op2) {
-	case PLUS:
-	case MINUS:
-		break;
-	default:
-		return false;
-	}
 	if (i1 <= loc) {
 		op1 = PLUS;
-	} else
+	} else {
 		op1 = equation[i1-1].token.operatr;
+	}
 	i = i1 - 1;
 f_outer:
 	b1 = i + 1;
@@ -283,28 +275,28 @@ f_inner:
 	}
 	ai = i;
 	aj = j;
-	if (whole_flag) {
-		if (flag1 = (b1 > i1)) {
-			b1 = i1;
-			save_k1 = equation[b1].token.constant;
-			equation[b1].token.constant = 1.0;
-		}
-		if (flag2 = (b2 > i2)) {
-			b2 = i2;
-			save_k2 = equation[b2].token.constant;
-			equation[b2].token.constant = 1.0;
-		}
+	save_k1 = 0.0;
+	save_k2 = 0.0;
+	flag1 = (whole_flag && b1 > i1);
+	if (flag1) {
+		b1 = i1;
+		save_k1 = equation[b1].token.constant;
+		equation[b1].token.constant = 1.0;
+	}
+	flag2 = (whole_flag && b2 > i2);
+	if (flag2) {
+		b2 = i2;
+		save_k2 = equation[b2].token.constant;
+		equation[b2].token.constant = 1.0;
 	}
 	same_flag = se_compare(&equation[b1], i - b1, &equation[b2], j - b2, &diff_sign);
-	if (whole_flag) {
-		if (flag1) {
-			equation[i1].token.constant = save_k1;
-			b1 += 2;
-		}
-		if (flag2) {
-			equation[i2].token.constant = save_k2;
-			b2 += 2;
-		}
+	if (flag1) {
+		equation[i1].token.constant = save_k1;
+		b1 += 2;
+	}
+	if (flag2) {
+		equation[i2].token.constant = save_k2;
+		b2 += 2;
 	}
 	if (same_flag) {
 		/* do the factor transformation */
@@ -318,10 +310,11 @@ more_power:
 			scratch[1].kind = OPERATOR;
 			scratch[1].token.operatr = DIVIDE;
 			len = 2;
-		} else
+		} else {
 			len = 0;
+		}
 		k = len;
-		blt(&scratch[len], &equation[b1], (ai - b1) * sizeof(*equation));
+		blt(&scratch[len], &equation[b1], (ai - b1) * sizeof(token_type));
 		len += ai - b1;
 		if (power != 1.0) {
 			for (; k < len; k++)
@@ -345,12 +338,12 @@ more_power:
 		scratch[len].token.operatr = TIMES;
 		len++;
 		k = len;
-		blt(&scratch[len], &equation[i1], (b1 - i1) * sizeof(*equation));
+		blt(&scratch[len], &equation[i1], (b1 - i1) * sizeof(token_type));
 		len += b1 - i1;
 		if (ai != i) {
 			l = len;
 			m = len + ai - b1;
-			blt(&scratch[len], &equation[b1], (i - b1) * sizeof(*equation));
+			blt(&scratch[len], &equation[b1], (i - b1) * sizeof(token_type));
 			len += i - b1;
 			if (b1 == i1 && i == e1) {
 				for (; l < len; l++)
@@ -377,20 +370,22 @@ more_power:
 		scratch[len].kind = CONSTANT;
 		if (op1 == MINUS) {
 			scratch[len].token.constant = -1.0;
-		} else
+		} else {
 			scratch[len].token.constant = 1.0;
+		}
 		len++;
-		blt(&scratch[len], &equation[i], (e1 - i) * sizeof(*equation));
+		blt(&scratch[len], &equation[i], (e1 - i) * sizeof(token_type));
 		len += e1 - i;
 		for (; k < len; k++)
 			scratch[k].level += 2;
 		scratch[len].level = level + 1;
 		scratch[len].kind = OPERATOR;
 		diff_sign ^= (op2 == MINUS);
-		if (diff_sign)
+		if (diff_sign) {
 			scratch[len].token.operatr = MINUS;
-		else
+		} else {
 			scratch[len].token.operatr = PLUS;
+		}
 		len++;
 		k = len;
 		if (aj != j) {
@@ -402,11 +397,11 @@ more_power:
 				error_huge();
 			}
 		}
-		blt(&scratch[len], &equation[i2], (b2 - i2) * sizeof(*equation));
+		blt(&scratch[len], &equation[i2], (b2 - i2) * sizeof(token_type));
 		len += b2 - i2;
 		if (aj != j) {
 			m = len + aj - b2;
-			blt(&scratch[len], &equation[b2], (j - b2) * sizeof(*equation));
+			blt(&scratch[len], &equation[b2], (j - b2) * sizeof(token_type));
 			len += j - b2;
 			l = m;
 			m++;
@@ -426,7 +421,7 @@ more_power:
 			scratch[len].token.constant = 1.0;
 			len++;
 		}
-		blt(&scratch[len], &equation[j], (e2 - j) * sizeof(*equation));
+		blt(&scratch[len], &equation[j], (e2 - j) * sizeof(token_type));
 		len += e2 - j;
 		for (; k < len; k++)
 			scratch[k].level += 2;
@@ -437,11 +432,11 @@ end_mess:
 		if (op1 == MINUS) {
 			equation[i1-1].token.operatr = PLUS;
 		}
-		blt(&equation[i2-1], &equation[e2], (*np - e2) * sizeof(*equation));
+		blt(&equation[i2-1], &equation[e2], (*np - e2) * sizeof(token_type));
 		*np -= n2 + 1;
-		blt(&equation[i1+len], &equation[e1], (*np - e1) * sizeof(*equation));
+		blt(&equation[i1+len], &equation[e1], (*np - e1) * sizeof(token_type));
 		*np += len - n1;
-		blt(&equation[i1], scratch, len * sizeof(*equation));
+		blt(&equation[i1], scratch, len * sizeof(token_type));
 		return true;
 	}
 	if (whole_flag)
@@ -496,11 +491,11 @@ end_mess:
 		goto f_inner;
 	if (se_compare(&equation[b1], ai - b1, &equation[b2], aj - b2, &diff_sign)) {
 		if (save_d1 > 0.0 || save_d2 > 0.0) {
-			if (save_d1 < 0.0)
+			if (save_d1 < 0.0) {
 				power = save_d2;
-			else if (save_d2 < 0.0)
+			} else if (save_d2 < 0.0) {
 				power = save_d1;
-			else {
+			} else {
 				power = min(save_d1, save_d2);
 				if (!diff_sign && (fmod(power, 1.0) != 0.0)) {
 					if (fmod(max(save_d1, save_d2) - power, 1.0) == 0.0) {
@@ -579,11 +574,12 @@ int		e1, e2;
 		scratch[1].kind = OPERATOR;
 		scratch[1].token.operatr = DIVIDE;
 		len = 2;
-	} else
+	} else {
 		len = 0;
+	}
 	k = len;
 	o = len + ai - b1;
-	blt(&scratch[len], &equation[b1], (i - b1) * sizeof(*equation));
+	blt(&scratch[len], &equation[b1], (i - b1) * sizeof(token_type));
 	len += i - b1;
 	if (b1 == i1 && i == e1) {
 		for (; k < len; k++)
@@ -594,7 +590,7 @@ int		e1, e2;
 	scratch[len].token.operatr = TIMES;
 	len++;
 	k = len;
-	blt(&scratch[len], &equation[i1], (b1 - i1) * sizeof(*equation));
+	blt(&scratch[len], &equation[i1], (b1 - i1) * sizeof(token_type));
 	len += b1 - i1;
 	scratch[len].level = level;
 	scratch[len].kind = CONSTANT;
@@ -604,7 +600,7 @@ int		e1, e2;
 		scratch[len].token.constant = 1.0;
 	}
 	len++;
-	blt(&scratch[len], &equation[i], (e1 - i) * sizeof(*equation));
+	blt(&scratch[len], &equation[i], (e1 - i) * sizeof(token_type));
 	len += e1 - i;
 	for (; k < len; k++)
 		scratch[k].level += 2;
@@ -613,14 +609,14 @@ int		e1, e2;
 	scratch[len].token.operatr = op2;
 	len++;
 	k = len;
-	blt(&scratch[len], &equation[i2], (b2 - i2) * sizeof(*equation));
+	blt(&scratch[len], &equation[i2], (b2 - i2) * sizeof(token_type));
 	len += b2 - i2;
 	if (len + (e2 - b2) + 2 * (i - ai) + 2 > n_tokens) {
 		error_huge();
 	}
 	n = len;
 	m = len + aj - b2;
-	blt(&scratch[len], &equation[b2], (j - b2) * sizeof(*equation));
+	blt(&scratch[len], &equation[b2], (j - b2) * sizeof(token_type));
 	len += j - b2;
 	l = m;
 	m++;
@@ -635,7 +631,7 @@ int		e1, e2;
 	scratch[len].token.operatr = MINUS;
 	len++;
 	m = len;
-	blt(&scratch[len], &equation[ai+1], (i - (ai + 1)) * sizeof(*equation));
+	blt(&scratch[len], &equation[ai+1], (i - (ai + 1)) * sizeof(token_type));
 	len += i - (ai + 1);
 	n = min_level(&equation[ai+1], i - (ai + 1));
 	n = scratch[l].level + 2 - n;
@@ -653,10 +649,10 @@ int		e1, e2;
 		scratch[len].kind = CONSTANT;
 		scratch[len].token.constant = -1.0;
 		len++;
-		blt(&scratch[len], &scratch[o], (i - ai) * sizeof(*equation));
+		blt(&scratch[len], &scratch[o], (i - ai) * sizeof(token_type));
 		len += i - ai;
 	}
-	blt(&scratch[len], &equation[j], (e2 - j) * sizeof(*equation));
+	blt(&scratch[len], &equation[j], (e2 - j) * sizeof(token_type));
 	len += e2 - j;
 	for (; k < len; k++)
 		scratch[k].level += 2;
@@ -751,13 +747,6 @@ int		*np, loc, i1, n1, i2, n2, level;
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
-	switch (op2) {
-	case TIMES:
-	case DIVIDE:
-		break;
-	default:
-		return false;
-	}
 	if (i1 <= loc) {
 		op1 = TIMES;
 	} else {
@@ -788,16 +777,6 @@ int		*np, loc, i1, n1, i2, n2, level;
 	if (se_compare(&equation[i1], i - i1, &equation[i2], j - i2, &diff_sign)) {
 		goto common_base;
 	}
-#if	false
-	if (i != e1 && se_compare(&equation[i1], e1 - i1, &equation[i2], j - i2, &diff_sign)) {
-		i = e1;
-		goto common_base;
-	}
-	if (j != e2 && se_compare(&equation[i1], i - i1, &equation[i2], e2 - i2, &diff_sign)) {
-		j = e2;
-		goto common_base;
-	}
-#endif
 	return false;
 
 common_base:
@@ -827,7 +806,7 @@ common_base:
 	if (*np + len > n_tokens) {
 		error_huge();
 	}
-	blt(&equation[e1+len], &equation[e1], (*np - e1) * sizeof(*equation));
+	blt(&equation[e1+len], &equation[e1], (*np - e1) * sizeof(token_type));
 	*np += len;
 	if (i == e1) {
 		for (k = i1; k < e1; k++)
@@ -841,7 +820,7 @@ common_base:
 	}
 	if (op1 == DIVIDE && !both_divide) {
 		equation[i1-1].token.operatr = TIMES;
-		blt(&equation[i+3], &equation[i+1], rlen1 * sizeof(*equation));
+		blt(&equation[i+3], &equation[i+1], rlen1 * sizeof(token_type));
 		i++;
 		equation[i].level = level;
 		equation[i].kind = CONSTANT;
@@ -864,10 +843,10 @@ common_base:
 		equation[i+1].kind = CONSTANT;
 		equation[i+1].token.constant = 1.0;
 	} else {
-		blt(&equation[i+1], &equation[j+len+1], (e2 - j - 1) * sizeof(*equation));
+		blt(&equation[i+1], &equation[j+len+1], (e2 - j - 1) * sizeof(token_type));
 	}
 	binary_parenthesize(equation, i + e2 - j, i);
-	blt(&equation[i2+len-1], &equation[e2+len], (*np - (e2 + len)) * sizeof(*equation));
+	blt(&equation[i2+len-1], &equation[e2+len], (*np - (e2 + len)) * sizeof(token_type));
 	*np -= n2 + 1;
 	return true;
 }
@@ -957,22 +936,15 @@ int		*np, loc, i1, n1, i2, n2, level;
 	int		len;
 	int		diff_sign;
 	int		all_divide;
-	token_type	temp;
 
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
-	switch (op2) {
-	case TIMES:
-	case DIVIDE:
-		break;
-	default:
-		return false;
-	}
 	if (i1 <= loc) {
 		op1 = TIMES;
-	} else
+	} else {
 		op1 = equation[i1-1].token.operatr;
+	}
 	for (i = i1 + 1;; i += 2) {
 		if (i >= e1)
 			return false;
@@ -1007,10 +979,7 @@ fp_outer:
 		}
 	}
 #endif
-	temp.level = 1;
-	temp.kind = CONSTANT;
-	temp.token.constant = 1.0;
-	if (se_compare(&equation[b1], i - b1, &temp, 1, &diff_sign)) {
+	if (se_compare(&equation[b1], i - b1, &one_token, 1, &diff_sign)) {
 		goto fp_outer;
 	}
 	j = start2;
@@ -1041,7 +1010,7 @@ fp_inner:
 			diff_sign = !diff_sign;
 		all_divide = (op1 == DIVIDE && diff_sign);
 		len = 0;
-		blt(&scratch[len], &equation[i1], (b1 - i1) * sizeof(*equation));
+		blt(&scratch[len], &equation[i1], (b1 - i1) * sizeof(token_type));
 		len += b1 - i1;
 		scratch[len].level = level + 1;
 		scratch[len].kind = CONSTANT;
@@ -1051,7 +1020,7 @@ fp_inner:
 			scratch[len].token.constant = 1.0;
 		}
 		len++;
-		blt(&scratch[len], &equation[i], (e1 - i) * sizeof(*equation));
+		blt(&scratch[len], &equation[i], (e1 - i) * sizeof(token_type));
 		len += e1 - i;
 		for (k = 0; k < len; k++)
 			scratch[k].level += 2;
@@ -1060,7 +1029,7 @@ fp_inner:
 		scratch[len].token.operatr = TIMES;
 		len++;
 		k = len;
-		blt(&scratch[len], &equation[i2], (b2 - i2) * sizeof(*equation));
+		blt(&scratch[len], &equation[i2], (b2 - i2) * sizeof(token_type));
 		len += b2 - i2;
 		scratch[len].level = level + 1;
 		scratch[len].kind = CONSTANT;
@@ -1070,7 +1039,7 @@ fp_inner:
 			scratch[len].token.constant = 1.0;
 		}
 		len++;
-		blt(&scratch[len], &equation[j], (e2 - j) * sizeof(*equation));
+		blt(&scratch[len], &equation[j], (e2 - j) * sizeof(token_type));
 		len += e2 - j;
 		for (; k < len; k++)
 			scratch[k].level += 2;
@@ -1089,7 +1058,7 @@ fp_inner:
 			len++;
 		}
 		k = len;
-		blt(&scratch[len], &equation[b1], (i - b1) * sizeof(*equation));
+		blt(&scratch[len], &equation[b1], (i - b1) * sizeof(token_type));
 		len += i - b1;
 		for (; k < len; k++)
 			scratch[k].level++;
@@ -1099,11 +1068,11 @@ fp_inner:
 		if (!all_divide && op1 == DIVIDE) {
 			equation[i1-1].token.operatr = TIMES;
 		}
-		blt(&equation[i2-1], &equation[e2], (*np - e2) * sizeof(*equation));
+		blt(&equation[i2-1], &equation[e2], (*np - e2) * sizeof(token_type));
 		*np -= n2 + 1;
-		blt(&equation[i1+len], &equation[e1], (*np - e1) * sizeof(*equation));
+		blt(&equation[i1+len], &equation[e1], (*np - e1) * sizeof(token_type));
 		*np += len - n1;
-		blt(&equation[i1], scratch, len * sizeof(*equation));
+		blt(&equation[i1], scratch, len * sizeof(token_type));
 		return true;
 	}
 	goto fp_inner;
