@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -12,90 +12,89 @@
  * The following structure is used for each Mathomatic command.
  */
 typedef	struct {
-	char	*name;		/* command name to be typed by user */
-	int	(*func)();	/* function that handles this command */
-				/* function is passed a char pointer and returns true if successful */
-	char	*usage;		/* command syntax */
-	char	*info;		/* one line description of command */
+	char	*name;			/* command name to be typed by user */
+	char	*secondary_name;	/* another name for this command */
+	int	(*func)();		/* function that handles this command */
+					/* function is passed a char pointer and returns true if successful */
+	char	*usage;			/* command syntax text */
+	char	*info;			/* one line description of command */
 } com_type;
 
 /*
  * The Mathomatic command table follows.  It should be in alphabetical order.
  */
 static com_type com_list[] = {
+/*	command name,	alternate name,		function,	usage,							information */
 #if	!LIBRARY
-{	"calculate",	calculate_cmd,	"[variable number-of-iterations]",			"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,	"[variable number-of-iterations]",			"Temporarily plug in values for variables and approximate." },
 #endif
-{	"clear",	clear_cmd,	"[equation-number-range]",				"Delete equations so equation space can be reused." },
-#if	!BASICS && !LIBRARY
-{	"code",		code_cmd,	"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-range]",	"Output C, Java, or Python code for the specified equations." },
-#endif
-{	"compare",	compare_cmd,	"equation-number [\"with\" equation-number]",		"Compare two equations to see if they are the same." },
-{	"copy",		copy_cmd,	"[equation-number-range]",				"Duplicate the specified equations." },
-{	"derivative",	derivative_cmd,	"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"clear",	NULL,			clear_cmd,	"[equation-numbers-or-ranges]",				"Delete expressions or equations so equation spaces can be reused." },
 #if	!LIBRARY
-{	"divide",	divide_cmd,	"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
+{	"code",		NULL,			code_cmd,	"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-range]",	"Output C, Java, or Python code for the specified equations." },
+#endif
+{	"compare",	NULL,			compare_cmd,	"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
+{	"copy",		NULL,			copy_cmd,	"[equation-number-range]",				"Duplicate the specified equation spaces." },
+{	"derivative",	"differentiate",	derivative_cmd,	"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+#if	!LIBRARY
+{	"divide",	NULL,			divide_cmd,	"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
 #if	(UNIX || CYGWIN) && !SECURE
-{	"edit",		edit_cmd,	"[file-name]",						"Edit your equations or an input file." },
+{	"edit",		NULL,			edit_cmd,	"[file-name]",						"Edit all equation spaces or an input file." },
 #endif
 #endif
-{	"eliminate",	eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current equation." },
-{	"factor",	factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor integers or equations.  For polynomial factoring, use simplify." },
+{	"eliminate",	NULL,			eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
+{	"extrema",	"extremum",		extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current expression." },
+{	"factor",	NULL,			factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Trivially factor integers or stored expressions." },
+{	"flist",	"fraction",		flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
+{	"help",		NULL,			help_cmd,	"[topic or command-name]",				"Short, built-in help." },
+{	"imaginary",	NULL,			imaginary_cmd,	"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
+{	"integrate",	"integral",		integrate_cmd,	"[\"definite\"] variable [order]",			"Symbolically integrate polynomials, order times." },
+{	"laplace",	NULL,			laplace_cmd,	"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
+{	"limit",	NULL,			limit_cmd,	"variable expression",					"Take the limit of the current expression as variable goes to expression." },
+{	"list",		NULL,			list_cmd,	"[\"export\" or \"maxima\"] [equation-number-range]",	"Display stored expressions in single line format." },
 #if	!LIBRARY
-{	"flist",	flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display equations in fraction format.  Option to factor integers." },
-#endif
-{	"help",		help_cmd,	"[topic or command-name]",				"Short, built-in help." },
-{	"imaginary",	imaginary_cmd,	"[equation-number]",					"Copy the imaginary part of an expression (see the \"real\" command)." },
-{	"integrate",	integrate_cmd,	"[\"definite\"] variable [order]",			"Symbolically integrate polynomials, order times." },
-{	"laplace",	laplace_cmd,	"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
-{	"limit",	limit_cmd,	"variable expression",					"Take the limit of the current equation as variable goes to expression." },
-{	"list",		list_cmd,	"[\"export\" or \"maxima\"] [equation-number-range]",	"Display equations in single line format.  Options for exportable format." },
-#if	!BASICS && !LIBRARY
-{	"nintegrate",	nintegrate_cmd,	"[\"trapezoidal\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
-{	"optimize",	optimize_cmd,	"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
+{	"nintegrate",	NULL,			nintegrate_cmd,	"[\"trapezoidal\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
+{	"optimize",	NULL,			optimize_cmd,	"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
 #endif
 #if	!LIBRARY
-{	"pause",	pause_cmd,	"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
+{	"pause",	NULL,			pause_cmd,	"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
 #endif
-{	"product",	product_cmd,	"variable start end [step]",				"Compute the product as variable goes from start to end." },
+{	"product",	NULL,			product_cmd,	"variable start end [step]",				"Compute the product as variable goes from start to end." },
 #if	READLINE
-{	"push",		push_cmd,	"[equation-number]",					"Push the current or specified equation into the readline history." },
+{	"push",		NULL,			push_cmd,	"[equation-number-range]",				"Push stored expressions into the readline history." },
 #endif
 #if	!LIBRARY
-{	"quit",		quit_cmd,	"",							"Terminate this program.  All equations in memory are lost." },
+{	"quit",		"exit",			quit_cmd,	"",							"Terminate this program without saving." },
 #endif
 #if	!SECURE && !LIBRARY
-{	"read",		read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
+{	"read",		NULL,			read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"real",		real_cmd,	"[equation-number]",					"Copy the real part of an expression (see the \"imaginary\" command)." },
-{	"replace",	replace_cmd,	"[\"constants\"] [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
+{	"real",		NULL,			real_cmd,	"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
+{	"replace",	NULL,			replace_cmd,	"[\"constants\"] [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
 #if	!LIBRARY
-{	"roots",	roots_cmd,	"root real-part imaginary-part",			"Display all the roots of a complex number." },
+{	"roots",	NULL,			roots_cmd,	"root real-part imaginary-part",			"Display all the roots of a complex number." },
 #endif
 #if	!SECURE
-{	"save",		save_cmd,	"file-name",						"Save all equations in a text file (restore with the \"read\" command)." },
+{	"save",		NULL,			save_cmd,	"file-name",						"Save all stored expressions and equations in a text file." },
 #endif
-{	"set",		set_cmd,	"[[\"no\"] option]",					"Set various session options." },
-{	"simplify",	simplify_cmd,	"[\"symbolic\"] [\"quick\"] [\"poly\"] [equation-number-range]",	"Completely simplify equations." },
-{	"sum",		sum_cmd,	"variable start end [step]",				"Compute the summation as variable goes from start to end." },
+{	"set",		NULL,			set_cmd,	"[[\"no\"] options]",					"Set or display various session options." },
+{	"simplify",	NULL,			simplify_cmd,	"[\"symbolic\"] [\"quick\"] [equation-number-range]",	"Completely simplify stored expressions." },
+{	"solve",	NULL,			solve_cmd,	"variable or \"0\"",					"Solve the current equation for a variable or for 0." },
+{	"sum",		NULL,			sum_cmd,	"variable start end [step]",				"Compute the summation as variable goes from start to end." },
 #if	!LIBRARY
-{	"tally",	tally_cmd,	"",							"Prompt for and add numerical entries.  Show running total and average." },
+{	"tally",	NULL,			tally_cmd,	"",							"Prompt for and add numerical entries, show running total and average." },
 #endif
-#if	!BASICS && !LIBRARY
-{	"taylor",	taylor_cmd,	"[variable]",						"Compute the Taylor series expansion of the current equation." },
-#endif
-{	"unfactor",	unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Expand expressions." },
+{	"taylor",	NULL,			taylor_cmd,	"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
+{	"unfactor",	"expand",		unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) stored expressions." },
 #if	!LIBRARY
-{	"version",	version_cmd,	"",							"Display version number and compile flags used." }
+{	"version",	NULL,			version_cmd,	"",							"Display version number and compile flags used." }
 #endif
 };
 
 /*
- * Process equation and expression input in Mathomatic.
+ * Process equation and expression input in Mathomatic, with no solving
+ * and no automatic calculation.
  *
- * Parse the equation or expression text in "cp"
- * and place in equation space "n".
+ * Parse the equation or expression text in "cp" and place in equation space "n".
  *
  * Return true if successful.
  */
@@ -121,8 +120,8 @@ char	*cp;
 /*
  * Process equation and expression input in Mathomatic.
  *
- * Parse the equation or expression text in "cp"
- * and solve the current equation or place in equation space "n".
+ * Parse the expression text in "cp" and solve the current equation for it
+ * or place it in equation space "n" if it is not a solve variable.
  *
  * Return true if successful.
  */
@@ -146,11 +145,18 @@ char	*cp;
 				}
 				return false;
 			}
+			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT
+			    && fmod(lhs[n][0].token.constant, 1.0) == 0.0
+			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
+				/* easy selecting of equation spaces */
+				n_lhs[n] = 0;
+				n = lhs[n][0].token.constant - 1;
+				goto return_ok;
+			}
 #if	!LIBRARY
 			/* the numerical input calculation */
 			for (i = 0; i < n_lhs[n]; i += 2) {
-				if (lhs[n][i].kind == VARIABLE
-				    && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
+				if (lhs[n][i].kind == VARIABLE && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
 					goto return_ok;		/* not numerical (contains a variable) */
 				}
 			}
@@ -182,7 +188,7 @@ return_ok:
 
 /*
  * Process a line of input to Mathomatic.
- * It may be a command, an equation, etc.
+ * It may be a command, an expression, an equation, etc.
  *
  * Return true if successful.
  */
@@ -210,15 +216,10 @@ char	*cp;
 		cp++;
 		switch (*cp) {
 		case '+':
-			cp++;
+		case '-':
 			i = decstrtol(cp, &cp1);
 			i = cur_equation + i;
 			break;
-		case '-':
-			cp++;
-			i = decstrtol(cp, &cp1);
-			i = cur_equation - i;
-			break;
 		default:
 			i = decstrtol(cp, &cp1) - 1;
 			break;
@@ -244,7 +245,7 @@ char	*cp;
 #if	!SECURE
 	/* handle shell escape */
 	if (*cp == '!') {
-		cp++;
+		cp = skip_space(cp + 1);
 		if (*cp) {
 			return(!shell_out(cp));
 		}
@@ -255,20 +256,20 @@ char	*cp;
 		return false;
 	}
 #endif
+	/* a quick way to get help */
+	if (*cp == '?') {
+		cp = skip_space(cp + 1);
+		input_column += (cp - cp_start);
+		return(help_cmd(cp));
+	}
 /* See if the string pointed to by "cp" is a command. */
 /* If so, execute it. */
 	cp1 = cp;
-	while (*cp1 && isascii(*cp1) && isalpha(*cp1))
+	while (*cp1 && isalpha(*cp1))
 		cp1++;
 	for (i = 0; i < ARR_CNT(com_list); i++) {
-		if (strlen(com_list[i].name) < 4) {
-			if ((cp1 - cp) != strlen(com_list[i].name))
-				continue;
-		} else {
-			if ((cp1 - cp) < 4)
-				continue;
-		}
-		if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0) {
+		if (((cp1 - cp) >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, cp1 - cp) == 0)
+		    || (com_list[i].secondary_name && (cp1 - cp) >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
 			if (my_strlcpy(buf2, cp1, sizeof(buf2)) >= sizeof(buf2)) {
@@ -308,7 +309,7 @@ char	*cp;
 #endif
 			/* remove trailing spaces from the command line */
 		        i1 = strlen(buf2) - 1;
-		        while (i1 >= 0 && isascii(buf2[i1]) && isspace(buf2[i1])) {
+		        while (i1 >= 0 && isspace(buf2[i1])) {
         		        buf2[i1] = '\0';
                 		i1--;
 			}
@@ -375,6 +376,7 @@ char	*cp;	/* pointer to variable name string */
 /*
  * The help command.
  */
+int
 help_cmd(cp)
 char	*cp;
 {
@@ -384,12 +386,14 @@ char	*cp;
 	int	row;
 
 	cp1 = cp;
-	while (*cp1 && !(isascii(*cp1) && isspace(*cp1)))
+	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	if (cp1 != cp) {
+		/* first, see if the argument matches any command names */
 		flag = false;
 		for (i = 0; i < ARR_CNT(com_list); i++) {
-			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0) {
+			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0
+			    || (com_list[i].secondary_name && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
 				fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
 				fprintf(gfp, "Usage: %s %s\n\n", com_list[i].name, com_list[i].usage);
 				flag = true;
@@ -427,8 +431,8 @@ char	*cp;
 			P("    Sum = (n-2)*180 degrees");
 			return true;
 		}
-		if (strncasecmp(cp, "equations", cp1 - cp) == 0) {
-			P("To enter an equation, simply type it in at the prompt.");
+		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
+			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 			P("    - negate");
 			P("    ! factorial (gamma function)");
@@ -438,8 +442,8 @@ char	*cp;
 			P("Operators in the same precedence level are evaluated left to right.\n");
 			P("Variables consist of any combination of letters, digits, underscores (_),");
 			P("and backslashes (\\).  Predefined constants and variables:\n");
-			P("    e# - the universal constant e (2.718281828...).");
-			P("    pi - the universal constant pi (3.1415926...).");
+			P("    e# - the universal constant e (2.71828182846...).");
+			P("    pi - the universal constant pi (3.14159265359...).");
 			P("    i# - imaginary number (square root of -1).");
 			P("    sign, sign1, sign2, sign3, ... - may be +1 or -1.");
 			P("    integer - may be any integer.");
@@ -447,9 +451,9 @@ char	*cp;
 			P("Absolute value notation \"|x|\" and \"+/-\" are understood.");
 			return true;
 		}
-		if (strncasecmp(cp, "commands", cp1 - cp) == 0 || is_all(cp)) {
+		if (is_all(cp)) {
 			for (i = 0, row = 1;; i++, row += 3) {
-				if (gfp == stdout && screen_rows && row >= (screen_rows - 3)) {
+				if (gfp == stdout && screen_rows && row >= (screen_rows - 4)) {
 					row = 1;
 					if (!pause_cmd(""))
 						return false;
@@ -464,10 +468,11 @@ char	*cp;
 			return true;
 		}
 	}
+	/* default help text: */
 	P("This help command is provided as a quick reference.");
 	P("Type \"help all\" for a summary of the all commands.");
 	P("Type \"help usage\" for syntax of all commands.");
-	P("Type \"help equations\" for help with entering equations.");
+	P("Type \"help expressions\" for help with entering expressions.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
 	P("\"help\" followed by a command name will give info on a command.\n");
 	P("Available commands:");
@@ -478,8 +483,7 @@ char	*cp;
 		for (; j > 0; j--)
 			fprintf(gfp, " ");
 	}
-	P("\n\nTo select an equation in memory, enter \"#\" followed by the equation number.");
-	P("For example, to select equation number 2, enter \"#2\" at the prompt.");
-	P("\nTo solve an equation, simply type the variable name at the prompt.");
+	P("\n\nTo select an equation in memory, just enter the equation number.");
+	P("To solve the current equation, type the variable name at the prompt.");
 	return true;
 }
