@@ -1,9 +1,9 @@
 /*
- * Algebraic manipulator integration routines.
+ * Mathomatic integration routines and commands.
  *
- * These are very low level, so they don't do much, mostly polynomials.
+ * These are very low level, so they don't do much, polynomials only.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -12,15 +12,15 @@ static int	integrate_sub();
 static int	laplace_sub();
 static int	inv_laplace_sub();
 
-static double	integrate_order;	/* order of integration */
+static double	integrate_order;	/* global order of integration */
 
 /*
  * Make variable "v" always raised to a power,
  * unless it is on the right side of a power operator.
  */
 make_powers(equation, np, v)
-token_type	*equation;	/* pointer to expression */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* pointer to beginning of equation side */
+int		*np;		/* pointer to length of equation side */
 long		v;		/* variable */
 {
 	int	i;
@@ -58,52 +58,55 @@ long		v;		/* variable */
 
 /*
  * Integration dispatch routine.
+ * Handles the level 1 additive operators.
  *
  * Return true if successful.
  */
 int
 int_dispatch(equation, np, v, func)
-token_type	*equation;
-int		*np;
-long		v;
-int		(*func)();	/* integration function to call */
+token_type	*equation;	/* pointer to beginning of equation side to integrate */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* integration variable */
+int		(*func)();	/* integration function to call for each term */
 {
 	int	i, j;
 
 	make_powers(equation, np, v);
-	if (level1_plus(equation, *np)) {
-		for (j = 0, i = 1;; i += 2) {
-			if (i >= *np || equation[i].level == 1) {
-				if (!(*func)(equation, np, j, i, v))
-					return false;
-				for (i = j + 1;; i += 2) {
-					if (i >= *np) {
-						return true;
-					}
-					if (equation[i].level == 1) {
-						j = i + 1;
-						break;
-					}
+	for (j = 0, i = 1;; i += 2) {
+		if (i >= *np) {
+			return((*func)(equation, np, j, i, v));
+		}
+		if (equation[i].level == 1
+		    && (equation[i].token.operatr == PLUS || equation[i].token.operatr == MINUS)) {
+			if (!(*func)(equation, np, j, i, v)) {
+				return false;
+			}
+			for (i = j + 1;; i += 2) {
+				if (i >= *np) {
+					return true;
+				}
+				if (equation[i].level == 1) {
+					j = i + 1;
+					break;
 				}
 			}
 		}
-		return true;
-	} else {
-		return((*func)(equation, np, 0, *np, v));
 	}
+	return true;
 }
 
 /*
- * Do the actual integration.
+ * Do the actual integration of a polynomial term.
  *
  * Return true if successful.
  */
 static int
 integrate_sub(equation, np, loc, eloc, v)
-token_type	*equation;
-int		*np;
-int		loc, eloc;
-long		v;
+token_type	*equation;	/* pointer to beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+int		loc;		/* beginning location of term */
+int		eloc;		/* end location of term */
+long		v;		/* variable of integration */
 {
 	int		i, j, k;
 	int		len;
@@ -112,6 +115,7 @@ long		v;
 	int		div_flag;
 
 	level = min_level(&equation[loc], eloc - loc);
+	/* determine if the term is a polynomial term in "v" */
 	for (i = loc, count = 0; i < eloc; i += 2) {
 		if (equation[i].kind == VARIABLE && equation[i].token.variable == v) {
 			count++;
@@ -229,7 +233,7 @@ integrate_cmd(cp)
 char	*cp;
 {
 	int		i, j;
-	long		v;
+	long		v = 0;
 	double		d1;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
@@ -239,7 +243,8 @@ char	*cp;
 	if (current_not_defined()) {
 		return false;
 	}
-	if (def_flag = (strcmp_tospace(cp, "definite") == 0)) {
+	def_flag = (strcmp_tospace(cp, "definite") == 0);
+	if (def_flag) {
 		cp = skip_param(cp);
 	}
 	i = next_espace();
@@ -279,7 +284,7 @@ char	*cp;
 	n1 = *nps;
 	for (d1 = 0; d1 < integrate_order; d1++) {
 		if (!int_dispatch(dest, &n1, v, integrate_sub)) {
-			error(_("Integration failed."));
+			error(_("Integration failed, not a polynomial."));
 			return false;
 		}
 		simp_loop(dest, &n1);
@@ -313,7 +318,7 @@ char	*cp;
 		blt(&dest[n1], scratch, n2 * sizeof(token_type));
 		n1 += n2;
 	}
-	simpa_side(dest, &n1, false, false);
+	simpa_side(dest, &n1, false);
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
@@ -325,7 +330,7 @@ char	*cp;
 }
 
 /*
- * Do the actual Laplace transformation.
+ * Do the actual Laplace transformation of a polynomial term.
  *
  * Return true if successful.
  */
@@ -414,7 +419,7 @@ long		v;
 }
 
 /*
- * Do the actual inverse Laplace transformation.
+ * Do the actual inverse Laplace transformation of a polynomial term.
  *
  * Return true if successful.
  */
@@ -491,7 +496,7 @@ laplace_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v;
+	long		v = 0;
 	int		inverse_flag;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -543,7 +548,7 @@ char	*cp;
 	} else {
 		if (!poly_in_v(dest, n1, v, false)
 		    || !int_dispatch(dest, &n1, v, laplace_sub)) {
-			error(_("Laplace failed."));
+			error(_("Laplace failed, not a polynomial."));
 			return false;
 		}
 	}
@@ -558,7 +563,7 @@ char	*cp;
 	return true;
 }
 
-#if	!BASICS && !LIBRARY
+#if	!LIBRARY
 /*
  * Numerical integrate command.
  */
@@ -566,16 +571,13 @@ int
 nintegrate_cmd(cp)
 char	*cp;
 {
-	long		v;
-	int		i, j, k;
-	int		i1, i2;
+	long		v = 0;
+	int		i, j, k, i1, i2;
 	int		level;
 	int		iterations;
 	int		first_size;
-	token_type	*ep;
-	int		trap_flag;
-	int		singularity;
-	token_type	*source, *dest;
+	int		trap_flag, singularity;
+	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
 
 	iterations = 1000;	/* must be even */
@@ -599,10 +601,7 @@ char	*cp;
 	if (trap_flag) {
 		cp = skip_param(cp);
 	}
-	if (*cp == '\0') {
-		if (!prompt_var(&v))
-			return false;
-	} else {
+	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
@@ -614,6 +613,9 @@ char	*cp;
 			error(_("Number of partitions must be a positive, even integer."));
 			return false;
 		}
+	} else {
+		if (!prompt_var(&v))
+			return false;
 	}
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
