@@ -1,8 +1,8 @@
 /*
- * Algebraic manipulator expression and equation display routines.
+ * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -54,8 +54,9 @@ int	color;
 		return;
 	if (color_flag) {
 		if (html_flag) {
-			if (cur_color >= 0)
+			if (cur_color >= 0) {
 				printf("</font>");
+			}
 			printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
 		} else {
 			if (bold_colors) {
@@ -77,13 +78,14 @@ default_color()
 }
 
 /*
- * Display the equation stored in equation space "n" in single line format.
+ * Display the expression or equation stored in equation space "n" in single line format.
+ * The equation space is not modified.
  *
  * Return true if successful.
  */
 int
 list1_sub(n, export_flag)
-int	n;		/* equation number */
+int	n;		/* equation space number */
 int	export_flag;	/* true for exportable format (readable by other math programs) */
 			/* 1 for Maxima, 2 for other */
 {
@@ -113,20 +115,26 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 }
 
 /*
- * Display the equation stored in equation space "n".
+ * Display the expression or equation stored in equation space "n".
  *
  * Return true if successful.
  */
 int
 list_sub(n)
-int	n;	/* equation number */
+int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return false;
 	if (groupall) {
 		group_sub(n);
+		if (factor_int_flag) {
+			factor_int_sub(n);
+		}
 		return flist_sub(n);
 	} else {
+		if (factor_int_flag) {
+			factor_int_sub(n);
+		}
 		return list1_sub(n, false);
 	}
 }
@@ -140,7 +148,7 @@ token_type	*p2;
 int		n2;
 {
 	if (debug_level >= level) {
-		printf(_("debug %d: "), level);
+		printf(_("level %d: "), level);
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
 			printf(" = ");
@@ -281,6 +289,7 @@ int	lang_code;	/* language code */
 /*
  * Display an expression in single line format.
  * Use color if available.
+ * The expression is not modified.
  *
  * Return number of characters output (excluding escape sequences).
  */
@@ -314,8 +323,11 @@ int		export_flag;	/* flag for exportable format (usually false) */
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
+			if (equation[i].token.constant == 0.0) {
+				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant <= 0.0) {
+				if (equation[i].token.constant < 0.0) {
 					len += fprintf(gfp, "(%.20g)", equation[i].token.constant);
 				} else {
 					len += fprintf(gfp, "%.20g", equation[i].token.constant);
@@ -349,11 +361,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				cp = " % ";
 				break;
 			case POWER:
-				if (starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
+				cp = "^";
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -380,8 +388,8 @@ int		export_flag;	/* flag for exportable format (usually false) */
  */
 char *
 list_equation(n, export_flag)
-int	n;		/* equation number */
-int	export_flag;
+int	n;		/* equation space number */
+int	export_flag;	/* flag for exportable format (usually false) */
 {
 	int	len;
 	char	*cp;
@@ -406,7 +414,6 @@ int	export_flag;
 	return cp;
 }
 
-#if	false
 /*
  * Store an expression in a text string.
  * String should be freed with free() when done.
@@ -430,7 +437,6 @@ int		export_flag;
 	list_string_sub(equation, n, cp, export_flag);
 	return cp;
 }
-#endif
 
 /*
  * Convert an expression to a text string and store in "string" if
@@ -474,8 +480,11 @@ int		export_flag;
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
+			if (equation[i].token.constant == 0.0) {
+				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant <= 0.0) {
+				if (equation[i].token.constant < 0.0) {
 					len += snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
 				} else {
 					len += snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
@@ -513,11 +522,7 @@ int		export_flag;
 				cp = " % ";
 				break;
 			case POWER:
-				if (starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
+				cp = "^";
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -551,11 +556,9 @@ int		n;		/* length of expression */
 	double	d;
 
 	for (i = 0; i < n; i++) {
-		if ((equation[i].kind == CONSTANT
-		    && fmod(equation[i].token.constant, 1.0) != 0.0)
-		    || (equation[i].kind == VARIABLE
-		    && var_is_const(equation[i].token.variable, &d))) {
-			return false;	/* non-integer constant */
+		if ((equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
+		    || (equation[i].kind == VARIABLE && var_is_const(equation[i].token.variable, &d))) {
+			return false;
 		}
 	}
 	return true;
@@ -564,19 +567,22 @@ int		n;		/* length of expression */
 /*
  * Display an equation as C, Java, or Python code.
  */
-list_c_equation(en, java_flag, int_flag)
-int	en;		/* equation number */
-int	java_flag;
+list_c_equation(en, language, int_flag)
+int	en;		/* equation space number */
+int	language;
 int	int_flag;	/* integer arithmetic flag */
 {
 	if (n_lhs[en] <= 0)
 		return;
-	list_code(lhs[en], &n_lhs[en], java_flag, int_flag);
+	list_code(lhs[en], &n_lhs[en], language, int_flag);
 	if (n_rhs[en]) {
 		fprintf(gfp, " = ");
-		list_code(rhs[en], &n_rhs[en], java_flag, int_flag);
+		list_code(rhs[en], &n_rhs[en], language, int_flag);
 	}
-	fprintf(gfp, ";\n");
+	if (language != 2) {
+		fprintf(gfp, ";");
+	}
+	fprintf(gfp, "\n");
 }
 
 /*
@@ -586,10 +592,10 @@ int	int_flag;	/* integer arithmetic flag */
  * Return length of output (number of characters).
  */
 int
-list_code(equation, np, java_flag, int_flag)
+list_code(equation, np, language, int_flag)
 token_type	*equation;	/* expression pointer */
 int		*np;		/* pointer to length of expression */
-int		java_flag;	/* if 1, generate Java code, if 2, generate Python code */
+int		language;	/* if 0, generate C code; if 1, generate Java code; if 2, generate Python code */
 int		int_flag;	/* integer arithmetic flag, should work with any language */
 {
 	int	i, j, k, i1, i2;
@@ -624,9 +630,9 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 								equation[i2].token.operatr = TIMES;
 								equation[i2+1] = equation[i2-1];
 							} else {
-								if (int_flag || java_flag > 1)
+								if (int_flag || language > 1)
 									break;
-								if (java_flag) {
+								if (language) {
 									cp = "Math.pow";
 								} else {
 									cp = "pow";
@@ -648,6 +654,9 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
+			if (equation[i].token.constant == 0.0) {
+				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
@@ -666,7 +675,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			len += fprintf(gfp, "%s", buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, java_flag + 1);
+			len += list_var(equation[i].token.variable, language + 1);
 			fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
@@ -688,7 +697,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				cp = " % ";
 				break;
 			case POWER:
-				if (int_flag || java_flag > 1) {
+				if (int_flag || language > 1) {
 					cp = " ** ";
 				} else {
 					cp = ", ";
@@ -713,14 +722,14 @@ int	cur_line;	/* current line */
 int	cur_pos;	/* current position in the current line on the screen */
 
 /*
- * Display an equation in fraction format.
+ * Display an expression or equation in multi-line fraction format.
  * Use color if available.
  *
  * Return true if successful.
  */
 int
 flist_sub(n)
-int	n;	/* equation number */
+int	n;	/* equation space number */
 {
 	int	i;
 	int	sind;
@@ -951,8 +960,9 @@ check_again:
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0)
+			if (equation[i].token.constant == 0.0) {
 				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
@@ -985,11 +995,7 @@ check_again:
 				cp = " % ";
 				break;
 			case POWER:
-				if (starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
+				cp = "^";
 				break;
 			case FACTORIAL:
 				cp = "!";
