@@ -1,22 +1,22 @@
 /*
- * Algebraic manipulator simplifying and polynomial routines.
+ * Mathomatic simplifying and polynomial routines.
  * Includes polynomial and smart division, polynomial factoring, etc.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
 
-/* For smart_div() and poly_div(): */
+#define	REMAINDER_IS_ZERO()	(n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
+
+/* These are a non-standard size and must only be used for temp storage: */
 token_type	divisor[DIVISOR_SIZE];
 int		n_divisor;
 token_type	quotient[DIVISOR_SIZE];
 int		n_quotient;
-/* These are a non-standard size and must only be used for temp storage. */
 token_type	gcd_divisor[DIVISOR_SIZE];
 int		len_d;
 
-static int	poly_in_v_sub();
 static int	do_gcd();
 static int	pf_recurse();
 static int	pf_sub();
@@ -24,6 +24,7 @@ static int	mod_recurse();
 static int	polydiv_recurse();
 static int	pdiv_recurse();
 static int	poly_div_sub();
+static void	save_factors();
 
 /*
  * Compare function for qsort(3).
@@ -36,44 +37,9 @@ sort_type	*p1, *p2;
 }
 
 /*
- * Return true if passed expression is a polynomial in
- * variable v.  The exponents may be anything, as long
- * as they don't contain the variable.
+ * Return true if passed expression is strictly a single polynomial term in v.
  */
-int
-poly_in_v(equation, n, v, allow_divides)
-token_type	*equation;	/* expression pointer */
-int		n;		/* expression length */
-long		v;		/* variable */
-int		allow_divides;	/* allow variable to be right of a divide (negative exponents) */
-{
-	int	i, j;
-	int	rv = true;
-
-	if (level1_plus(equation, n)) {
-		for (i = 1, j = 0;; i += 2) {
-			if (i >= n || equation[i].level == 1) {
-				if (!poly_in_v_sub(&equation[j], i - j, v, allow_divides)) {
-					rv = false;
-					break;
-				}
-				j = i + 1;
-			}
-			if (i >= n)
-				break;
-		}
-	} else {
-		if (!poly_in_v_sub(equation, n, v, allow_divides))
-			rv = false;
-	}
-	return rv;
-}
-
-/*
- * Return true if passed expression is strictly a single
- * polynomial term in v.
- */
-static int
+static inline int
 poly_in_v_sub(equation, n, v, allow_divides)
 token_type	*equation;
 int		n;
@@ -123,17 +89,45 @@ int		allow_divides;
 	return true;
 }
 
+/*
+ * Return true if passed expression is a polynomial in
+ * variable v.  The exponents may be anything, as long
+ * as they don't contain the variable.
+ */
+int
+poly_in_v(equation, n, v, allow_divides)
+token_type	*equation;	/* expression pointer */
+int		n;		/* expression length */
+long		v;		/* variable */
+int		allow_divides;	/* allow variable to be right of a divide (negative exponents) */
+{
+	int	i, j;
+
+	for (i = 1, j = 0;; i += 2) {
+		if (i >= n || (equation[i].level == 1
+		    && (equation[i].token.operatr == PLUS || equation[i].token.operatr == MINUS))) {
+			if (!poly_in_v_sub(&equation[j], i - j, v, allow_divides)) {
+				return false;
+			}
+			j = i + 1;
+		}
+		if (i >= n)
+			break;
+	}
+	return true;
+}
+
 /*
  * Factor polynomials.
  * Factors repeated factor polynomials (like (x+1)^5)
  * and symbolic factor polynomials (like (x+a)*(x+b)).
  *
- * Return true if equation side was modified.
+ * Return true if equation side was modified (factored).
  */
 int
 poly_factor(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
 {
 	return pf_recurse(equation, np, 0, 1);
 }
@@ -161,8 +155,6 @@ int		*np, loc, level;
 		modified = pf_sub(equation, np, loc, len, level);
 		break;
 	}
-	if (modified)
-		return true;
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
 			modified |= pf_recurse(equation, np, i, level + 1);
@@ -178,9 +170,10 @@ int		*np, loc, level;
 
 /*
  * Polynomial factoring subroutine.
- * It can't factor everything,
- * but it usually can factor polynomials
+ * It can't factor everything, but it usually can factor polynomials
  * if it will make the expression size smaller.
+ *
+ * Return true if expression was modified (factored).
  */
 static int
 pf_sub(equation, np, loc, len, level)
@@ -215,7 +208,7 @@ int		*np, loc, len, level;
 		}
 		return(modified || single_modified);
 	}
-/* First factor polynomials with repeated factors. */
+/* First factor polynomials with repeated factors: */
 	for (count = 1;; count++) {
 		blt(trhs, &equation[loc1], len * sizeof(token_type));
 		n_trhs = len;
@@ -224,8 +217,7 @@ int		*np, loc, len, level;
 			goto skip_factor;
 		last_v = 0;
 		for (vc = 0; vc < ARR_CNT(va); vc++) {
-			v1 = -1;
-			for (i = 0; i < n_trhs; i += 2) {
+			for (i = 0, cnt = 0, v1 = -1; i < n_trhs; i += 2) {
 				if (trhs[i].kind == VARIABLE && trhs[i].token.variable > last_v) {
 					if (v1 == -1 || trhs[i].token.variable < v1) {
 						v1 = trhs[i].token.variable;
@@ -307,7 +299,7 @@ int		*np, loc, len, level;
 		}
 		modified = true;
 	}
-/* Now try and factor polynomials with symbolic factors. */
+/* Now factor polynomials with symbolic factors by grouping: */
 	if (!modified) {
 		last_v = 0;
 next_v:
@@ -328,12 +320,17 @@ next_v:
 				break;
 			}
 			last_v = v;
+			/* make sure there is more than one "v" raised to the highest power: */
 			if (find_greatest_power(trhs, n_trhs, &v, &d, &j, &k, &div_flag) <= 1) {
 				continue;
 			}
 			blt(tlhs, trhs, n_trhs * sizeof(token_type));
 			n_tlhs = n_trhs;
-			factorv(tlhs, &n_tlhs, v);
+			/* do the grouping: */
+			while (factor_plus(tlhs, &n_tlhs, v, 0.0)) {
+				simp_loop(tlhs, &n_tlhs);
+			}
+			/* extract the highest power group: */
 			if (find_greatest_power(tlhs, n_tlhs, &v, &d, &j, &k, &div_flag) != 1) {
 				continue;
 			}
@@ -388,6 +385,7 @@ skip_factor:
 	return(modified || single_modified);
 }
 
+static void
 save_factors(equation, np, loc1, len, level)
 token_type	*equation;
 int		*np, loc1, len, level;
@@ -413,8 +411,8 @@ int		*np, loc1, len, level;
 
 /*
  * Remove level 1 trivial factors and divides from "tlhs".
- * Return true if result is (or will be after unfactoring)
- * a level 1 plus type expression.
+ *
+ * Return true if result is a level 1 plus type expression.
  * If this returns false, nothing is removed.
  */
 int
@@ -430,7 +428,7 @@ remove_factors()
 	do {
 		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 0);
 	} while (uf_power(tlhs, &n_tlhs));
-	for (i = 0, j = 0, k = 0;; i++) {
+	for (i = 1, j = 0, k = 0;; i += 2) {
 		if (i >= n_tlhs) {
 			if (plus_flag && !divide_flag) {
 				if (k > 0)
@@ -445,31 +443,37 @@ remove_factors()
 			side_debug(3, tlhs, n_tlhs);
 			return true;
 		}
-		if (tlhs[i].kind == OPERATOR) {
-			if (tlhs[i].level == 1) {
-				op = tlhs[i].token.operatr;
-				if (op == PLUS || op == MINUS) {
-					plus_flag = true;
-					continue;
-				}
-				if (op != TIMES && op != DIVIDE)
-					return false;
-				if (plus_flag && !divide_flag) {
-					if (k > 0)
-						j--;
-					blt(&scratch[k], &tlhs[j], (i - j) * sizeof(token_type));
-					k += i - j;
-				}
-				plus_flag = false;
-				divide_flag = (op == DIVIDE);
-				j = i + 1;
-			} else if (tlhs[i].level == 2) {
-				switch (tlhs[i].token.operatr) {
-				case PLUS:
-				case MINUS:
-					plus_flag = true;
-				}
+		op = tlhs[i].token.operatr;
+		switch (tlhs[i].level) {
+		case 1:
+			switch (op) {
+			case PLUS:
+			case MINUS:
+				plus_flag = true;
+				continue;
+			case TIMES:
+			case DIVIDE:
+				break;
+			default:
+				return false;
 			}
+			if (plus_flag && !divide_flag) {
+				if (k > 0)
+					j--;
+				blt(&scratch[k], &tlhs[j], (i - j) * sizeof(token_type));
+				k += i - j;
+			}
+			plus_flag = false;
+			divide_flag = (op == DIVIDE);
+			j = i + 1;
+			break;
+		case 2:
+			switch (op) {
+			case PLUS:
+			case MINUS:
+				plus_flag = true;
+			}
+			break;
 		}
 	}
 }
@@ -489,12 +493,13 @@ long		*vp;
 	int	count;
 
 	for (count = 1;; count++) {
-		if (!poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
+		switch (poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
+		case 0:
+			/* divide failed */
 			return 0;
-		}
-		if (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0) {
-			/* remainder is 0, success! */
-			break;
+		case 2:
+			/* Total success!  Remainder is zero. */
+			return count;
 		}
 		/* swap trhs[] (remainder) and gcd_divisor[] */
 		if (len_d > n_tokens || n_trhs > DIVISOR_SIZE)
@@ -506,11 +511,11 @@ long		*vp;
 		n_trhs = len_d;
 		len_d = i;
 	}
-	return count;
 }
 
 /*
  * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
+ *
  * Return true if successful.
  * Return the GCD in "trhs".
  * Return "larger"/GCD in "tlhs".
@@ -550,9 +555,8 @@ long		v;
 			return 0;
 		blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 		len_d = n_tlhs;
-		if (!poly_div(larger, llen, gcd_divisor, len_d, &v)
-		    || n_trhs != 1 || trhs[0].kind != CONSTANT || trhs[0].token.constant != 0.0) {
-			error("Polynomial GCD found, but larger divide failed.");
+		if (poly_div(larger, llen, gcd_divisor, len_d, &v) != 2) {
+			debug_string(0, "Polynomial GCD found, but larger divide failed.");
 			return 0;
 		}
 	}
@@ -568,6 +572,7 @@ long		v;
 
 /*
  * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
+ *
  * Return true if successful.
  * Return "larger"/GCD in "tlhs".
  * Return "smaller"/GCD in "trhs".
@@ -603,9 +608,8 @@ long		v;
 	if (count > 1) {
 		if (!level1_plus(gcd_divisor, len_d))
 			return 0;
-		if (!poly_div(smaller, slen, gcd_divisor, len_d, &v)
-		    || n_trhs != 1 || trhs[0].kind != CONSTANT || trhs[0].token.constant != 0.0) {
-			error("Polynomial GCD found, but smaller divide failed.");
+		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
+			debug_string(0, "Polynomial GCD found, but smaller divide failed.");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -616,18 +620,15 @@ long		v;
 		len_d = n_tlhs;
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
-		if (!poly_div(larger, llen, tlhs, n_tlhs, &v)
-		    || n_trhs != 1 || trhs[0].kind != CONSTANT || trhs[0].token.constant != 0.0) {
-			error("Polynomial GCD found, but larger divide failed.");
+		if (poly_div(larger, llen, tlhs, n_tlhs, &v) != 2) {
+			debug_string(0, "Polynomial GCD found, but larger divide failed.");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
 		n_trhs = len_d;
 	} else {
 		n_trhs = 1;
-		trhs[0].level = 1;
-		trhs[0].kind = CONSTANT;
-		trhs[0].token.constant = 1.0;
+		trhs[0] = one_token;
 	}
 	uf_simp(tlhs, &n_tlhs);
 	uf_simp(trhs, &n_trhs);
@@ -645,16 +646,13 @@ is_integer_expr(equation, n)
 token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 {
-	int	i2;
+	int	i;
 	double	d;
 
-	for (i2 = 0; i2 < n; i2++) {
-		if ((equation[i2].kind == OPERATOR
-		    && equation[i2].token.operatr == DIVIDE)
-		    || (equation[i2].kind == CONSTANT
-		    && fmod(equation[i2].token.constant, 1.0))
-		    || (equation[i2].kind == VARIABLE
-		    && var_is_const(equation[i2].token.variable, &d))) {
+	for (i = 0; i < n; i++) {
+		if ((equation[i].kind == OPERATOR && equation[i].token.operatr == DIVIDE)
+		    || (equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
+		    || (equation[i].kind == VARIABLE && var_is_const(equation[i].token.variable, &d))) {
 			return false;
 		}
 	}
@@ -683,8 +681,7 @@ int		*np, loc, level;
 	int	modified = false;
 	int	i, j, k;
 	int	i1, i2, i3, i4, i5;
-	int	op;
-	int	last_op2;
+	int	op, last_op2;
 	int	len1, len2, len3;
 	long	v;
 	int	diff_sign;
@@ -709,9 +706,7 @@ int		*np, loc, level;
 			}
 			len1 = k - (i + 1);
 			last_op2 = 0;
-			for (j = loc;; j++) {
-				if (j >= *np || equation[j].level < level)
-					break;
+			for (j = loc; j < *np && equation[j].level >= level; j++) {
 				if (equation[j].level == level && equation[j].kind == OPERATOR) {
 					last_op2 = equation[j].token.operatr;
 					continue;
@@ -721,9 +716,7 @@ int		*np, loc, level;
 				}
 				last_op2 = MODULUS;
 				op = 0;
-				for (k = j + 1;; k += 2) {
-					if (k >= *np || equation[k].level <= level)
-						break;
+				for (k = j + 1; k < *np && equation[k].level > level; k += 2) {
 					if (equation[k].level == (level + 1)) {
 						op = equation[k].token.operatr;
 						i1 = k;
@@ -813,19 +806,16 @@ int		*np, loc, level;
 				v = 0;
 				if (poly_div(&equation[j], len2, &equation[i+1], len1, &v)) {
 					if (is_integer_expr(tlhs, n_tlhs)) {
+						if ((*np + (n_trhs - len2)) > n_tokens)
+							error_huge();
 #if	!SILENT
-						for (k = 0; k < n_tlhs; k += 2) {
-							if (tlhs[k].kind == VARIABLE) {
-								printf(_("Modulus simplification made some variables integer only.\n"));
-								break;
-							}
+						if (var_count(tlhs, n_tlhs)) {
+							printf(_("Modulus simplification made some variables integer only.\n"));
 						}
 #endif
 						for (k = 0; k < n_trhs; k++)
 							trhs[k].level += level;
-						if ((*np + (n_trhs - len2)) > n_tokens)
-							error_huge();
-						blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(*equation));
+						blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
 						*np += n_trhs - len2;
 						blt(&equation[j], trhs, n_trhs * sizeof(token_type));
 						return true;
@@ -845,7 +835,7 @@ int		*np, loc, level;
  * and denominator by the GCD.
  */
 int
-polydiv_simp(equation, np)
+poly_gcd_simp(equation, np)
 token_type	*equation;
 int		*np;
 {
@@ -880,9 +870,7 @@ int		*np, loc, level;
 			}
 			len1 = k - (i + 1);
 			last_op2 = 0;
-			for (j = loc;; j++) {
-				if (j >= *np || equation[j].level < level)
-					break;
+			for (j = loc; j < *np && equation[j].level >= level; j++) {
 				if (equation[j].level == level && equation[j].kind == OPERATOR) {
 					last_op2 = equation[j].token.operatr;
 					continue;
@@ -905,14 +893,14 @@ store_code:
 					if (((*np + (n_trhs - len2)) > n_tokens)
 					    || ((*np + (n_trhs - len2) + (n_tlhs - len1)) > n_tokens))
 						error_huge();
-					blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(*equation));
+					blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
 					*np += n_trhs - len2;
 					if (i > j)
 						i += n_trhs - len2;
 					blt(&equation[j], trhs, n_trhs * sizeof(token_type));
-					blt(&equation[i+n_tlhs+1], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(*equation));
+					blt(&equation[i+n_tlhs+1], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(token_type));
 					*np += n_tlhs - len1;
-					blt(&equation[i+1], tlhs, n_tlhs * sizeof(*equation));
+					blt(&equation[i+1], tlhs, n_tlhs * sizeof(token_type));
 					debug_string(0, _("Found polynomial Greatest Common Divisor.  Division simplified."));
 					return true;
 				}
@@ -934,6 +922,7 @@ store_code:
 /*
  * This routine is a division simplifier for equation sides.
  * Check for divides and do polynomial and smart division.
+ *
  * Return true if expression was simplified.
  */
 int
@@ -960,15 +949,11 @@ int		*np, loc, level, code;
 {
 	int	modified = false;
 	int	i, j, k;
-	int	op, op2;
-	int	last_op2;
-	int	len1, len2;
-	int	real_len1;
+	int	op, op2, last_op2;
+	int	len1, len2, real_len1;
 	long	v;
 	int	rv;
-	int	flag;
-	int	power_flag;
-	int	rem_zero;
+	int	flag, power_flag, zero_remainder;
 
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
@@ -978,9 +963,7 @@ int		*np, loc, level, code;
 			}
 			real_len1 = k - (i + 1);
 			last_op2 = 0;
-			for (j = loc;; j++) {
-				if (j >= *np || equation[j].level < level)
-					break;
+			for (j = loc; j < *np && equation[j].level >= level; j++) {
 				if (equation[j].level == level && equation[j].kind == OPERATOR) {
 					last_op2 = equation[j].token.operatr;
 					continue;
@@ -990,11 +973,10 @@ int		*np, loc, level, code;
 				}
 				last_op2 = DIVIDE;
 				op = 0;
-				for (k = j + 1;; k += 2) {
-					if (k >= *np || equation[k].level <= level)
-						break;
-					if (equation[k].level == level + 1)
+				for (k = j + 1; k < *np && equation[k].level > level; k += 2) {
+					if (equation[k].level == (level + 1)) {
 						op = equation[k].token.operatr;
+					}
 				}
 				if (op != PLUS && op != MINUS) {
 					continue;
@@ -1004,9 +986,7 @@ int		*np, loc, level, code;
 				power_flag = false;
 				op = 0;
 				op2 = 0;
-				for (k = i + 2;; k += 2) {
-					if (k >= *np || equation[k].level <= level)
-						break;
+				for (k = i + 2; k < *np && equation[k].level > level; k += 2) {
 					if (equation[k].level == level + 3) {
 						switch (equation[k].token.operatr) {
 						case PLUS:
@@ -1017,8 +997,7 @@ int		*np, loc, level, code;
 						op = equation[k].token.operatr;
 					} else if (equation[k].level == level + 1) {
 						if (equation[k].token.operatr == POWER
-						    && (op == PLUS || op == MINUS
-						    || (op == TIMES && op2 == PLUS))) {
+						    && (op == PLUS || op == MINUS || (op == TIMES && op2 == PLUS))) {
 							power_flag = true;
 							len1 = k - (i + 1);
 						}
@@ -1035,9 +1014,8 @@ next_thingy:
 				} else {
 					rv = smart_div(&equation[j], len2, &equation[i+1], len1);
 				}
-				rem_zero = (rv > 0 && n_trhs == 1 && trhs[0].kind == CONSTANT
-				    && trhs[0].token.constant == 0.0);
-				if (power_flag && !rem_zero) {
+				zero_remainder = (rv > 0 && REMAINDER_IS_ZERO());
+				if (power_flag && !zero_remainder) {
 					rv = 0;
 				}
 				if (rv > 0 && (n_tlhs + 2 + n_trhs + len1) <= n_tokens) {
@@ -1140,7 +1118,8 @@ next_thingy:
 
 /*
  * Do polynomial division.
- * Returns true if successful.
+ *
+ * Returns non-zero if successful, 2 if remainder is 0.
  * Quotient is returned in "tlhs" and remainder in "trhs".
  *
  * If "*vp" is 0, automatically select the best base variable,
@@ -1215,9 +1194,10 @@ long		*vp;
 			return false;
 	}
 #if	!SILENT
+	/* Display debugging info. */
 	if (debug_level >= 3) {
 		list_var(*vp, 0);
-		printf(_("poly_div starts using variable (%s):\n"), var_str);
+		printf(_("poly_div() starts using variable (%s):\n"), var_str);
 		side_debug(3, trhs, n_trhs);
 		side_debug(3, tlhs, n_tlhs);
 	}
@@ -1237,6 +1217,7 @@ long		*vp;
 	/* Initialize the quotient. */
 	n_quotient = 1;
 	quotient[0] = zero_token;
+	/* Store the divisor. */
 	if (n_tlhs > ARR_CNT(divisor))
 		return false;
 	blt(divisor, tlhs, n_tlhs * sizeof(token_type));
@@ -1244,9 +1225,10 @@ long		*vp;
 	sum_size = n_trhs + n_quotient;
 	/* Loop until polynomial division is finished. */
 	for (;;) {
-		sign = PLUS;
 		if (t1 > 0 && trhs[t1-1].token.operatr == MINUS)
 			sign = MINUS;
+		else
+			sign = PLUS;
 		if (t2 > 0 && divisor[t2-1].token.operatr == MINUS) {
 			if (sign == MINUS)
 				sign = PLUS;
@@ -1323,8 +1305,8 @@ long		*vp;
 			n_tlhs = n_quotient;
 			side_debug(3, tlhs, n_tlhs);
 			side_debug(3, trhs, n_trhs);
-			if (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
-				return true;
+			if (REMAINDER_IS_ZERO())
+				return 2;
 			if ((n_trhs + n_quotient) >= (sum_size /* - (sum_size / 10) */)) {
 				if ((n_trhs + 1) > sum_size && n_trhs > n_divisor)
 					return -2;
@@ -1368,16 +1350,14 @@ int		len1;		/* length of dividend */
 token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
 {
-#define	MAX_TRIES	100
-
 	int		i, j, k;
 	int		t1, len_t1;
-	int		t2, len_t2;
+	int		t2, len_t2 = 0;
 	int		sign;
 	int		old_n_quotient;
 	int		trhs_size;
 	int		term_size, term_count;
-	int		term_pos, skip_terms[MAX_TRIES];
+	int		term_pos, skip_terms[100];
 	int		skip_count;
 	token_type	*qp;
 	int		q_size;
@@ -1393,11 +1373,12 @@ int		len2;		/* length of divisor */
 	/* Do the basic unfactoring and simplification of the dividend and divisor. */
 	uf_simp_no_repeat(trhs, &n_trhs);
 	uf_simp_no_repeat(tlhs, &n_tlhs);
+	/* Display debugging info. */
 	debug_string(3, "smart_div() starts:");
 	side_debug(3, trhs, n_trhs);
 	side_debug(3, tlhs, n_tlhs);
 	/* Find which divisor term to use. */
-	for (i = 0, j = 0, k = 0, len_t2 = 0, flag = false;; i++) {
+	for (i = 0, j = 0, k = 0, flag = false;; i++) {
 		if (i >= n_tlhs || (tlhs[i].kind == OPERATOR && tlhs[i].level == 1
 		    && (tlhs[i].token.operatr == PLUS || tlhs[i].token.operatr == MINUS))) {
 			dcount++;
@@ -1410,8 +1391,7 @@ int		len2;		/* length of divisor */
 			}
 			flag = false;
 			j = i + 1;
-		} else if (tlhs[i].kind == VARIABLE
-		    && (tlhs[i].token.variable & VAR_MASK) > SIGN) {
+		} else if (tlhs[i].kind == VARIABLE && (tlhs[i].token.variable & VAR_MASK) > SIGN) {
 			flag = true;
 		}
 		if (i >= n_tlhs)
@@ -1428,8 +1408,7 @@ int		len2;		/* length of divisor */
 	n_divisor = n_tlhs;
 try_one:
 	trhs_size = n_trhs;
-	skip_count = 0;
-	for (count = 0;;) {
+	for (skip_count = 0, count = 0;;) {
 		sum_size = n_trhs + n_quotient;
 		for (term_count = 1, q_size = 0;; term_count++) {
 			if (!get_term(trhs, n_trhs, term_count, &t1, &len_t1))
@@ -1479,8 +1458,7 @@ try_one:
 			}
 end_div:
 			if (dcount > 1) {
-				i = n_quotient + n_trhs;
-				if (i >= trhs_size + 1) {
+				if (n_quotient + n_trhs >= trhs_size + 1) {
 					return false;
 				}
 			}
@@ -1569,7 +1547,7 @@ end_div2:
 		side_debug(3, trhs, n_trhs);
 		uf_tsimp(trhs, &n_trhs);
 		side_debug(4, trhs, n_trhs);
-		if (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
+		if (REMAINDER_IS_ZERO())
 			goto end_div2;
 		if (dcount > 1) {
 			if ((n_trhs + n_quotient) >= sum_size) {
@@ -1688,10 +1666,8 @@ long		*vp1;		/* variable pointer */
 
 	last_v = 0;
 	for (vc = 0; vc < ARR_CNT(va); vc++) {
-		v1 = -1;
-		for (i = 0; i < n1; i += 2) {
-			if (p1[i].kind == VARIABLE
-			    && p1[i].token.variable > last_v) {
+		for (i = 0, cnt = 0, v1 = -1; i < n1; i += 2) {
+			if (p1[i].kind == VARIABLE && p1[i].token.variable > last_v) {
 				if (v1 == -1 || p1[i].token.variable < v1) {
 					v1 = p1[i].token.variable;
 					cnt = 1;
