@@ -3,6 +3,20 @@
  *
  * Copyright (C) 2005 George Gesslein II.
  *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *
  * Usage: matho-pascal [number-of-lines]
  */
 
@@ -74,8 +88,8 @@ main(int argc, char *argv[])
 	for (i = 0; i < lines; i++) {
 		array[i] = (long double *) malloc(sizeof(long double) * lines);
 		if (array[i] == NULL) {
-			fprintf(stderr, "%s: malloc() error!\n", prog_name);
-			exit(1);
+			fprintf(stderr, "%s: Not enough memory.\n", prog_name);
+			exit(2);
 		}
 	}
 	calculate_triangle();
