@@ -1,11 +1,24 @@
 /*
  * Generate consecutive prime numbers using a modified sieve algorithm
  * that doesn't use much memory by using a windowing sieve buffer.
- * Works on up to 19 digit numbers.
+ * Works with up to 19 digit numbers.
  *
- * This is C or C++ program.
  * Copyright (C) 2006 George Gesslein II.
  *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *
  * Usage: matho-primes [start [stop]] ["pal" [base]]
  *
  * If "pal" is specified, display only palindromic primes of base "base",
@@ -14,6 +27,7 @@
  * Changes:
  *
  * 11/22/05 - converted everything to long doubles.
+ * 3/25/06 - made primes buffer variable size.
  */
 
 #include <stdio.h>
@@ -31,17 +45,19 @@
 #define	false	0
 
 #ifndef	max
-#define max(a,b)    (((a) > (b)) ? (a) : (b))
+#define max(a, b)    (((a) > (b)) ? (a) : (b))
 #endif
 
 #ifndef	min
-#define min(a,b)    (((a) < (b)) ? (a) : (b))
+#define min(a, b)    (((a) < (b)) ? (a) : (b))
 #endif
 
+#define	ARR_CNT(a)	(sizeof(a)/sizeof(a[0]))	/* returns the number of elements in an array */
+
 #define	MAX_K_INTEGER	1.0e19L		/* 19 digits for long doubles */
 
 /* memory usage in bytes; can be set to any size; the larger, the faster */
-#define BUFFER_SIZE	1000000
+#define BUFFER_SIZE	2000000
 #if	BUFFER_SIZE >= (INT_MAX / 2)
 #error	BUFFER_SIZE too big!
 #endif
@@ -54,21 +70,22 @@ int	get_long_double_int(char *cp, long double *dp);
 long double strtold(const char *nptr, char **endptr);
 
 long double start_value, number, end_value;
-long double sq[] = {
+long double sq[] = {			/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
 	 2, 6, 4, 6, 8, 4, 2, 4,
 	 2, 4, 8, 6, 4, 6, 2, 4,
 	 6, 2, 6, 6, 4, 2, 4, 6,
 	 2, 6, 4, 2, 4, 2,10, 2
-};
+};	/* sum of all numbers = 210 = (2*3*5*7) */
 
 long double	last_prime = -3.0;
 
 int		pal_flag, twin_flag;
 long double	pal_base = 10;
 
-char	prime[BUFFER_SIZE];	/* the boolean sieve array for the current window of numbers */
+int	buffer_size = BUFFER_SIZE;	/* for variable size prime buffer */
+char	*prime;				/* the boolean sieve array for the current window of numbers */
 
 char	*prog_name = "matho-primes";
 
@@ -79,7 +96,7 @@ main(int argc, char *argv[])
 	extern int	optind;
 
 	int		i;
-	char		buf[100];
+	char		buf[1000];
 
         prog_name = strdup(basename(argv[0]));
 	start_value = -1.0;
@@ -190,6 +207,12 @@ get2:
 			}
 		}
 	}
+	buffer_size = (int) min(BUFFER_SIZE, (end_value - start_value) + 1.0L);
+	prime = (char *) malloc(buffer_size);
+	if (prime == NULL) {
+		fprintf(stderr, "%s: Not enough memory.\n", prog_name);
+		exit(2);
+	}
 	generate_primes();
 	exit(0);
 }
@@ -208,15 +231,15 @@ elim_factor(long double arg)
 		d = 2.0;
 	d *= arg;
 	d -= start_value;
-	if (d >= BUFFER_SIZE)
+	if (d >= buffer_size)
 		return;
 	i = (int) d;
 	assert(i >= 0);
-	if (arg >= BUFFER_SIZE) {
+	if (arg >= buffer_size) {
 		prime[i] = false;
 	} else {
 		j = (int) arg;
-		for (; i < BUFFER_SIZE; i += j) {
+		for (; i < buffer_size; i += j) {
 			prime[i] = false;
 		}
 	}
@@ -228,24 +251,22 @@ elim_factor(long double arg)
 void
 generate_primes()
 {
-	int		n;
-	int		j;
+	int		n, j;
 	long double	count, ii, vv;
 
-	for (count = 0; count < number; start_value += BUFFER_SIZE) {
+	for (count = 0; count < number; start_value += buffer_size) {
 		if (start_value > end_value) {
 			return;
 		}
-		vv = 1.0 + sqrtl(min(start_value + (long double) BUFFER_SIZE, end_value));
-		for (n = 0; n < BUFFER_SIZE; n++)
-			prime[n] = true;
+		memset(prime, true, buffer_size);	/* set the prime array to all true */
+		vv = 1.0 + sqrtl(min(start_value + (long double) buffer_size, end_value));
 		elim_factor(2.0L);
 		elim_factor(3.0L);
 		elim_factor(5.0L);
 		elim_factor(7.0L);
 		ii = 1.0;
 		for (;;) {
-			for (j = 0; j < 48; j++) {
+			for (j = 0; j < ARR_CNT(sq); j++) {
 				ii += sq[j];
 				elim_factor(ii);
 			}
@@ -253,13 +274,13 @@ generate_primes()
 				break;
 		}
 		/* display the batch of generated prime numbers */
-		for (n = 0; n < BUFFER_SIZE && count < number; n++) {
+		for (n = 0; n < buffer_size && count < number; n++) {
 			if (prime[n]) {
 				ii = start_value + n;
 				if (ii > end_value) {
 					return;
 				}
-				if (ii != 0.0 && ii != 1.0) {
+				if (ii > 1.0) {
 					if (pal_flag && !test_pal(ii, pal_base))
 						continue;
 					if (twin_flag) {
