@@ -1,27 +1,24 @@
 /*
- * Algebraic manipulator simplifying routines.
+ * Mathomatic simplify routines.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
 
-#define	MAX_COMPARE_TERMS	400	/* Max no. of terms on same level that can be compared.  Increases stack usage. */
+#define	MAX_COMPARE_TERMS	400	/* Max no. of terms on same level that can be compared, increases stack usage. */
 
-static int	sp2_sub();
-static int	sp_sub();
 static int	org_recurse();
 static int	const_recurse();
 static int	compare_recurse();
 static int	order_recurse();
-static void	swap();
-static int	calc();
 
 /*
- * Minimally and very basically simplify an equation space.
+ * Quickly and very basically simplify an equation space.
+ * No factoring is done.
  */
 simp_sub(n)
-int	n;	/* equation number */
+int	n;	/* equation space number to simplify */
 {
 	simp_loop(lhs[n], &n_lhs[n]);
 	simp_loop(rhs[n], &n_rhs[n]);
@@ -31,8 +28,8 @@ int	n;	/* equation number */
  * For quick, mid-range simplification of an expression.
  */
 simp_side(equation, np)
-token_type	*equation;	/* expression */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
 {
 	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
 }
@@ -41,22 +38,22 @@ int		*np;		/* pointer to length of expression */
  * This function is the mid-range simplifier used by the solver.
  */
 simps_side(equation, np, zsolve)
-token_type	*equation;	/* expression */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
 int		zsolve;		/* true if we are solving for zero */
 {
 	do {
 		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
-	} while (super_factor(equation, np, false));
+	} while (super_factor(equation, np, 0));
 }
 
 /*
  * This function is used by the "factor" command.
  */
 simpv_side(equation, np, v)
-token_type	*equation;
-int		*np;
-long		v;
+token_type	*equation;	/* pointer to the beginning of equation side to factor */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* variable to factor */
 {
 	if (*np == 0)
 		return;
@@ -82,8 +79,7 @@ int		*np;
 }
 
 /*
- * Try to eliminate "i#" from an equation side,
- * by converting "i#*(b^.5)" to "(-1*b)^.5".
+ * Try to eliminate "i#" from an equation side by converting "i#*(b^.5)" to "(-1*b)^.5".
  */
 simp_i(equation, np)
 token_type	*equation;
@@ -118,9 +114,8 @@ int		*np;
 		do {
 			do {
 				do {
-					do {
-						organize(equation, np);
-					} while (combine_constants(equation, np, false));
+					organize(equation, np);
+					combine_constants(equation, np, false);
 				} while (elim_k(equation, np));
 			} while (simp_pp(equation, np));
 		} while (factor_power(equation, np));
@@ -129,25 +124,30 @@ int		*np;
 }
 
 /*
- * Main simplify code used by the "simplify" command.
+ * Main simplify code used by the simplify command.
  * "tlhs" and "trhs" are wiped out.
+ * This is the slowest and most thorough simplify of all.
  */
-simpa_side(equation, np, poly_flag, quick_flag)
-token_type	*equation;	/* pointer to equation side to simplify */
+simpa_side(equation, np, quick_flag)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of the equation side */
-int		poly_flag;	/* "simplify poly" option */
 int		quick_flag;	/* "simplify quick" option */
 {
-	int	i;
-	int	flag;
-	jmp_buf	save_save;
+	int		flag, poly_flag = false;
+	jmp_buf		save_save;
 
 	if (*np <= 1)	/* no need to simplify a constant or a variable */
 		return;
 	partial_flag = 2;
+	simp_loop(equation, np);
+	poly_factor(equation, np);
+#if	false
 	do {
-		simp_ssub(equation, np, MATCH_ANY, 0.0, false, true, 1);
+		simp_ssub(equation, np, 0L, 0.0, false, true, 1);
 	} while (uf_power(equation, np));
+#else
+	simpb_side(equation, np, true, 1);
+#endif
 	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
 	rationalize(equation, np);
 	unsimp_power(equation, np);
@@ -157,7 +157,7 @@ int		quick_flag;	/* "simplify quick" option */
 	do {
 		elim_loop(equation, np);
 	} while (mod_simp(equation, np));
-/* Here we try to simplify out unnecessary negative constants and imaginary numbers */
+	/* Here we try to simplify out unnecessary negative constants and imaginary numbers */
 	simp_i(equation, np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
@@ -169,7 +169,7 @@ int		quick_flag;	/* "simplify quick" option */
 		do {
 			do {
 				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-			} while (polydiv_simp(equation, np));
+			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
 	unsimp_power(equation, np);
@@ -178,10 +178,6 @@ int		quick_flag;	/* "simplify quick" option */
 	uf_pplus(equation, np);
 	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
 	uf_times(equation, np);
-	do {
-		elim_loop(equation, np);
-	} while (factor_power(equation, np));
-	uf_times(equation, np);
 	uf_pplus(equation, np);
 	factorv(equation, np, IMAGINARY);
 	uf_power(equation, np);
@@ -189,7 +185,7 @@ int		quick_flag;	/* "simplify quick" option */
 		do {
 			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
 		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, true));
+	} while (super_factor(equation, np, 1));
 	if (quick_flag) {
 do_quick:
 		uf_tsimp(equation, np);
@@ -197,46 +193,44 @@ do_quick:
 		n_tlhs = *np;
 		blt(tlhs, equation, n_tlhs * sizeof(token_type));
 		blt(save_save, jmp_save, sizeof(jmp_save));
-		if ((i = setjmp(jmp_save)) != 0) {
+		if (setjmp(jmp_save) != 0) {
 			blt(jmp_save, save_save, sizeof(jmp_save));
-			if (i == 13) {
-				longjmp(jmp_save, i);
-			}
+			/* restore the original expression */
 			*np = n_tlhs;
 			blt(equation, tlhs, n_tlhs * sizeof(token_type));
 			goto do_quick;
 		}
-		uf_repeat(equation, np);
-		uf_tsimp(equation, np);
-		uf_repeat(equation, np);
-		uf_tsimp(equation, np);
+		/* expand powers of 2 and higher, might result in error_huge() trap */
+		do {
+			uf_repeat(equation, np);
+		} while (uf_tsimp(equation, np));
 		blt(jmp_save, save_save, sizeof(jmp_save));
 	}
 	partial_flag = false;
 	simpb_side(equation, np, true, 1);
-	debug_string(1, "Simplify result before applying polynomial operations:");
+	debug_string(1, "Simplified result before applying polynomial operations:");
 	side_debug(1, equation, *np);
-	flag = false;
-try_again:
-	/* divide top and bottom of fractions by any polynomial GCD found */
-	while (polydiv_simp(equation, np)) {
-		flag = false;
-		simpb_side(equation, np, true, 1);
-	}
-	if (!flag) {
-		/* factor polynomials */
-		while (poly_factor(equation, np)) {
-			flag = true;
+	for (flag = false;;) {
+		/* divide top and bottom of fractions by any polynomial GCD found */
+		if (poly_gcd_simp(equation, np)) {
+			flag = false;
 			simpb_side(equation, np, true, 1);
 		}
-		if (flag)
-			goto try_again;
-	}
-	/* simplify algebraic fractions even more with polynomial and smart division */
-	if (div_remainder(equation, np, poly_flag, quick_flag)) {
-		flag = false;
-		simpb_side(equation, np, true, 1);
-		goto try_again;
+		if (!flag) {
+			/* factor polynomials */
+			if (poly_factor(equation, np)) {
+				flag = true;
+				simpb_side(equation, np, true, 1);
+				continue;
+			}
+		}
+		/* simplify algebraic fractions with polynomial and smart division */
+		if (div_remainder(equation, np, poly_flag, quick_flag)) {
+			flag = false;
+			simpb_side(equation, np, true, 1);
+			continue;
+		}
+		break;
 	}
 	partial_flag = true;
 	simp_constant_power(equation, np);
@@ -252,9 +246,7 @@ try_again:
 	while (div_remainder(equation, np, poly_flag, quick_flag)) {
 		simpb_side(equation, np, true, 3);
 	}
-	while (poly_factor(equation, np)) {
-		simpb_side(equation, np, true, 3);
-	}
+	poly_factor(equation, np);
 	simpb_side(equation, np, true, 2);
 }
 
@@ -284,13 +276,14 @@ sort_type	*p1, *p2;
 }
 
 /*
+ * Beauty simplifier.
  * Neat simplify and factor routine.
  * Factors variables in order.
  * "sign" variables first, then by frequency.
  */
 simpb_side(equation, np, power_flag, fc_level)
-token_type	*equation;	/* expression pointer */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
 int		power_flag;	/* factor powers flag */
 int		fc_level;	/* factor constants code */
 {
@@ -303,10 +296,8 @@ int		fc_level;	/* factor constants code */
 	uf_allpower(equation, np);
 	last_v = 0;
 	for (vc = 0; vc < ARR_CNT(va); vc++) {
-		v1 = -1;
-		for (i = 0; i < *np; i += 2) {
-			if (equation[i].kind == VARIABLE
-			    && equation[i].token.variable > last_v) {
+		for (i = 0, cnt = 0, v1 = -1; i < *np; i += 2) {
+			if (equation[i].kind == VARIABLE && equation[i].token.variable > last_v) {
 				if (v1 == -1 || equation[i].token.variable < v1) {
 					v1 = equation[i].token.variable;
 					cnt = 1;
@@ -321,25 +312,23 @@ int		fc_level;	/* factor constants code */
 		va[vc].v = v1;
 		va[vc].count = cnt;
 	}
-	if (vc == 0) {
-		goto skip_factor;
-	}
-	qsort((char *) va, vc, sizeof(*va), vcmp);
-	simp2_divide(equation, np, va[0].v, fc_level);
-	for (i = 1; i < vc; i++) {
-		if (factor_divide(equation, np, va[i].v, 0.0))
-			simp2_divide(equation, np, va[i].v, fc_level);
-	}
-	simp2_divide(equation, np, 0L, fc_level);
-	for (i = 0; i < vc; i++) {
-		while (factor_plus(equation, np, va[i].v, 0.0)) {
+	if (vc) {
+		qsort((char *) va, vc, sizeof(*va), vcmp);
+		simp2_divide(equation, np, va[0].v, fc_level);
+		for (i = 1; i < vc; i++) {
+			if (factor_divide(equation, np, va[i].v, 0.0))
+				simp2_divide(equation, np, va[i].v, fc_level);
+		}
+		simp2_divide(equation, np, 0L, fc_level);
+		for (i = 0; i < vc; i++) {
+			while (factor_plus(equation, np, va[i].v, 0.0)) {
+				simp2_divide(equation, np, 0L, fc_level);
+			}
+		}
+		while (factor_plus(equation, np, MATCH_ANY, 0.0)) {
 			simp2_divide(equation, np, 0L, fc_level);
 		}
 	}
-	while (factor_plus(equation, np, MATCH_ANY, 0.0)) {
-		simp2_divide(equation, np, 0L, fc_level);
-	}
-skip_factor:
 	simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
 }
 
@@ -348,8 +337,8 @@ skip_factor:
  * Just does constant simplification.
  */
 elim_loop(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
 {
 	side_debug(6, equation, *np);
 	do {
@@ -369,21 +358,16 @@ int		*np;
 
 /*
  * Configurable high level simplify routine.
- *
- * Returns the number of times the simplify loop was repeated.
  */
-int
 simp_ssub(equation, np, v, d, power_flag, times_flag, fc_level)
-token_type	*equation;	/* equation side to simplify */
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
-long		v;		/* variable to factor */
+long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
 double		d;		/* factor expressions raised to the power of this if v */
 int		power_flag;	/* factor power flag */
 int		times_flag;	/* factor times flag */
 int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
-	int	count = -1;
-
 	do {
 		do {
 			do {
@@ -392,7 +376,6 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 						do {
 							do {
 								do {
-									count++;
 									elim_loop(equation, np);
 								} while (simp2_power(equation, np));
 							} while (times_flag && factor_times(equation, np));
@@ -402,7 +385,6 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 			} while (factor_divide(equation, np, v, d));
 		} while (factor_plus(equation, np, v, d));
 	} while (power_flag && factor_power(equation, np));
-	return count;
 }
 
 /*
@@ -412,8 +394,8 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
  */
 int
 simp_loop(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
 {
 	int	i;
 	int	rv = false;
@@ -440,7 +422,7 @@ int		*np;
  * If v is IMAGINARY, do complex number simplification.
  */
 factorv(equation, np, v)
-token_type	*equation;	/* equation side */
+token_type	*equation;	/* pointer to the beginning of equation side to factor */
 int		*np;		/* pointer to equation side length */
 long		v;		/* variable to factor */
 {
@@ -753,28 +735,7 @@ int		*np;
 	return modified;
 }
 
-/*
- * Simplify constant^(constant*x).
- *
- * Return true if equation side was modified.
- */
-int
-simp_constant_power(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	modified = false;
-	int	i;
-
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			modified |= sp2_sub(equation, np, i);
-		}
-	}
-	return modified;
-}
-
-static int
+static inline int
 sp2_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -798,7 +759,7 @@ int		*np, i;
 		if (*np + 2 > n_tokens) {
 			error_huge();
 		}
-		blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
+		blt(&equation[j+2], &equation[j], (*np - j) * sizeof(token_type));
 		*np += 2;
 		equation[j+1].level = level + 1;
 		equation[j+1].kind = CONSTANT;
@@ -815,12 +776,12 @@ int		*np, i;
 }
 
 /*
- * Convert x^-y to 1/(x^y).
+ * Simplify constant^(constant*x).
  *
  * Return true if equation side was modified.
  */
 int
-simp2_power(equation, np)
+simp_constant_power(equation, np)
 token_type	*equation;
 int		*np;
 {
@@ -829,13 +790,13 @@ int		*np;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == POWER) {
-			modified |= sp_sub(equation, np, i);
+			modified |= sp2_sub(equation, np, i);
 		}
 	}
 	return modified;
 }
 
-static int
+static inline int
 sp_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -875,7 +836,7 @@ int		*np, i;
 		k++;
 		for (i = k; i < j; i++)
 			equation[i].level++;
-		blt(&equation[k+2], &equation[k], (*np - k) * sizeof(*equation));
+		blt(&equation[k+2], &equation[k], (*np - k) * sizeof(token_type));
 		*np += 2;
 		equation[k].level = level;
 		equation[k].kind = CONSTANT;
@@ -889,6 +850,27 @@ int		*np, i;
 	return false;
 }
 
+/*
+ * Convert x^-y to 1/(x^y).
+ *
+ * Return true if equation side was modified.
+ */
+int
+simp2_power(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	modified = false;
+	int	i;
+
+	for (i = 1; i < *np; i += 2) {
+		if (equation[i].token.operatr == POWER) {
+			modified |= sp_sub(equation, np, i);
+		}
+	}
+	return modified;
+}
+
 /*
  * Combine two or more constants on the same level of parentheses.
  * If "iflag" is false, don't produce imaginary numbers.
@@ -897,151 +879,19 @@ int		*np, i;
  */
 int
 combine_constants(equation, np, iflag)
-token_type	*equation;	/* expression pointer */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
 int		iflag;		/* produce imaginary numbers flag */
 {
 	return const_recurse(equation, np, 0, 1, iflag);
 }
 
-static int
-const_recurse(equation, np, loc, level, iflag)
-token_type	*equation;
-int		*np, loc, level, iflag;
-{
-	int		loc1, old_loc;
-	int		const_count = 0;
-	int		op, zero = 0;
-	int		modified = false;
-	double		d, d1, d2, numerator, denominator;
-	complexs	cv, p;
-
-	old_loc = loc;
-	for (;; loc++) {
-beginning:
-		if (loc >= *np || equation[loc].level < level) {
-			if (loc - old_loc == 1)	/* decrement the level of parentheses if only one constant left */
-				equation[old_loc].level = max(level - 1, 1);
-			return modified;
-		}
-		if (equation[loc].level > level) {
-			modified |= const_recurse(equation, np, loc, level + 1, iflag);
-			for (; loc < *np && equation[loc].level > level; loc++)
-				;
-			goto beginning;
-		}
-		if (equation[loc].kind == CONSTANT) {
-			if (const_count == 0) {
-				loc1 = loc;
-				const_count++;
-				continue;
-			}
-			op = equation[loc-1].token.operatr;
-			d = equation[loc1].token.constant;
-			d2 = equation[loc].token.constant;
-			if (calc((loc1 <= old_loc) ? &zero : &equation[loc1-1].token.operatr, &d, op, d2)) {
-				if (op == POWER && !domain_check) {
-					if (!f_to_fraction(d2, &numerator, &denominator)) {	/* if irrational power */
-						if (!iflag || (preserve_roots && !in_calc_cmd))
-							return modified;
-						cv.re = d;
-						cv.im = 0.0;
-						p.re = d2;
-						p.im = 0.0;
-						cv = complex_pow(cv, p);
-						if (*np + 2 > n_tokens) {
-							error_huge();
-						}
-						blt(&equation[loc1+2], &equation[loc1], (*np - loc1) * sizeof(*equation));
-						*np += 2;
-						equation[loc1].level = level;
-						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = cv.re;
-						equation[loc1+1].level = level;
-						equation[loc1+1].kind = OPERATOR;
-						equation[loc1+1].token.operatr = PLUS;
-						equation[loc].level = level + 1;
-						equation[loc].kind = VARIABLE;
-						equation[loc].token.variable = IMAGINARY;
-						equation[loc+1].level = level + 1;
-						equation[loc+1].kind = OPERATOR;
-						equation[loc+1].token.operatr = TIMES;
-						equation[loc+2].level = level + 1;
-						equation[loc+2].kind = CONSTANT;
-						equation[loc+2].token.constant = cv.im;
-						return true;
-					}
-					errno = 0;
-					d1 = pow(-d, d2);
-					check_err();
-					if (!always_positive(denominator)) {
-						if (!always_positive(numerator)) {
-							d1 = -d1;
-						}
-						d = d1;
-						goto not_imaginary;
-					}
-					if (!iflag)
-						return modified;
-					if (*np + 2 > n_tokens) {
-						error_huge();
-					}
-					blt(&equation[loc1+2], &equation[loc1], (*np - loc1) * sizeof(*equation));
-					*np += 2;
-					if (d2 == 0.5) {
-						equation[loc1].level = level + 1;
-						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = -d;
-						equation[loc1+1].level = level + 1;
-						equation[loc1+1].kind = OPERATOR;
-						equation[loc1+1].token.operatr = POWER;
-						equation[loc].level = level + 1;
-						equation[loc].kind = CONSTANT;
-						equation[loc].token.constant = d2;
-						equation[loc+1].level = level;
-						equation[loc+1].kind = OPERATOR;
-						equation[loc+1].token.operatr = TIMES;
-						equation[loc+2].level = level;
-						equation[loc+2].kind = VARIABLE;
-						equation[loc+2].token.variable = IMAGINARY;
-					} else {
-						equation[loc1].level = level;
-						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = d1;
-						equation[loc1+1].level = level;
-						equation[loc1+1].kind = OPERATOR;
-						equation[loc1+1].token.operatr = TIMES;
-						equation[loc].level = level + 1;
-						equation[loc].kind = VARIABLE;
-						equation[loc].token.variable = IMAGINARY;
-						equation[loc+1].level = level + 1;
-						equation[loc+1].kind = OPERATOR;
-						equation[loc+1].token.operatr = POWER;
-						equation[loc+2].level = level + 1;
-						equation[loc+2].kind = CONSTANT;
-						equation[loc+2].token.constant = d2 * 2.0;
-					}
-					return true;
-				} else {
-not_imaginary:
-					equation[loc1].token.constant = d;
-					modified = true;
-					domain_check = false;
-					blt(&equation[loc-1], &equation[loc+1], (*np - (loc + 1)) * sizeof(*equation));
-					*np -= 2;
-					loc -= 2;
-				}
-			}
-		}
-	}
-}
-
 /*
  * Do the arithmetic.
  *
  * Return true if successful.
  */
-static int
+static inline int
 calc(op1p, k1p, op2, k2)
 int	*op1p;	/* pointer to operator 1, sometimes modified */
 double	*k1p;	/* pointer to operand 1 and where to store the result */
@@ -1117,14 +967,12 @@ double	k2;	/* operand 2 */
 			break;
 		}
 		if (*k1p == 0.0 && k2 < 0.0) {
-#if	!SILENT
 			error(_("Divide by zero error."));
-#endif
 			longjmp(jmp_save, 2);
 		}
 		domain_check = true;
 		if (*k1p == 0.0 && k2 == 0.0) {
-			debug_string(0, _("Warning: 0 raised to the power of 0."));
+			debug_string(0, _("Warning: 0^0 encountered, might be considered undefined."));
 			d = 1.0;	/* some people don't know 0^0 */
 		} else {
 			d = pow(*k1p, k2);
@@ -1164,6 +1012,149 @@ double	k2;	/* operand 2 */
 	return true;
 }
 
+static int
+const_recurse(equation, np, loc, level, iflag)
+token_type	*equation;
+int		*np, loc, level, iflag;
+{
+	int		loc1, old_loc;
+	int		const_count = 0;
+	int		op, zero = 0;
+	int		modified = false;
+	double		d, d1, d2, numerator, denominator;
+	complexs	cv, p;
+
+	loc1 = old_loc = loc;
+	for (;; loc++) {
+beginning:
+		if (loc >= *np || equation[loc].level < level) {
+			if (loc - old_loc == 1)	/* decrement the level of parentheses if only one constant left */
+				equation[old_loc].level = max(level - 1, 1);
+			return modified;
+		}
+		if (equation[loc].level > level) {
+			modified |= const_recurse(equation, np, loc, level + 1, iflag);
+			for (; loc < *np && equation[loc].level > level; loc++)
+				;
+			goto beginning;
+		}
+		if (equation[loc].kind == CONSTANT) {
+			if (const_count == 0) {
+				loc1 = loc;
+				const_count++;
+				continue;
+			}
+			op = equation[loc-1].token.operatr;
+			d = equation[loc1].token.constant;
+			d2 = equation[loc].token.constant;
+			if (calc((loc1 <= old_loc) ? &zero : &equation[loc1-1].token.operatr, &d, op, d2)) {
+				if (op == POWER && !domain_check) {
+					if (!f_to_fraction(d2, &numerator, &denominator)) {	/* if irrational power */
+						if (!iflag || (preserve_roots && !in_calc_cmd))
+							return modified;
+						cv.re = d;
+						cv.im = 0.0;
+						p.re = d2;
+						p.im = 0.0;
+						cv = complex_pow(cv, p);
+						if (*np + 2 > n_tokens) {
+							error_huge();
+						}
+						blt(&equation[loc1+2], &equation[loc1], (*np - loc1) * sizeof(token_type));
+						*np += 2;
+						equation[loc1].level = level;
+						equation[loc1].kind = CONSTANT;
+						equation[loc1].token.constant = cv.re;
+						loc1++;
+						equation[loc1].level = level;
+						equation[loc1].kind = OPERATOR;
+						equation[loc1].token.operatr = PLUS;
+						level++;
+						equation[loc].level = level;
+						equation[loc].kind = VARIABLE;
+						equation[loc].token.variable = IMAGINARY;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = OPERATOR;
+						equation[loc].token.operatr = TIMES;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = CONSTANT;
+						equation[loc].token.constant = cv.im;
+						return true;
+					}
+					errno = 0;
+					d1 = pow(-d, d2);
+					check_err();
+					if (!always_positive(denominator)) {
+						if (!always_positive(numerator)) {
+							d1 = -d1;
+						}
+						d = d1;
+						goto not_imaginary;
+					}
+					if (!iflag)
+						return modified;
+					if (*np + 2 > n_tokens) {
+						error_huge();
+					}
+					blt(&equation[loc1+2], &equation[loc1], (*np - loc1) * sizeof(token_type));
+					*np += 2;
+					if (d2 == 0.5) {
+						equation[loc1].level = level + 1;
+						equation[loc1].kind = CONSTANT;
+						equation[loc1].token.constant = -d;
+						loc1++;
+						equation[loc1].level = level + 1;
+						equation[loc1].kind = OPERATOR;
+						equation[loc1].token.operatr = POWER;
+						equation[loc].level = level + 1;
+						equation[loc].kind = CONSTANT;
+						equation[loc].token.constant = d2;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = OPERATOR;
+						equation[loc].token.operatr = TIMES;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = VARIABLE;
+						equation[loc].token.variable = IMAGINARY;
+					} else {
+						equation[loc1].level = level;
+						equation[loc1].kind = CONSTANT;
+						equation[loc1].token.constant = d1;
+						loc1++;
+						equation[loc1].level = level;
+						equation[loc1].kind = OPERATOR;
+						equation[loc1].token.operatr = TIMES;
+						level++;
+						equation[loc].level = level;
+						equation[loc].kind = VARIABLE;
+						equation[loc].token.variable = IMAGINARY;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = OPERATOR;
+						equation[loc].token.operatr = POWER;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = CONSTANT;
+						equation[loc].token.constant = d2 * 2.0;
+					}
+					return true;
+				} else {
+not_imaginary:
+					equation[loc1].token.constant = d;
+					modified = true;
+					domain_check = false;
+					blt(&equation[loc-1], &equation[loc+1], (*np - (loc + 1)) * sizeof(token_type));
+					*np -= 2;
+					loc -= 2;
+				}
+			}
+		}
+	}
+}
+
 /*
  * Eliminate operations that involve constants that can be simplified.
  * Fix addition/subtraction of negative or infinity constants.
@@ -1663,8 +1654,9 @@ no_op2:
 
 /*
  * Take out meaningless "sign" variables and negative constants.
- * Simplify imaginary numbers raised to the power of some constants
- * (including simple division ((i#^-1) -> (-1*i#))).
+ * Simplify imaginary numbers raised to the power of some constants,
+ * including simple division (1/i# -> -1*i#).
+ *
  * Return true if equation side was modified.
  */
 int
@@ -1747,8 +1739,7 @@ int		*np;
 					}
 				}
 			} else {
-				if (equation[i-1].level == level
-				    && equation[i-1].kind == VARIABLE) {
+				if (equation[i-1].level == level && equation[i-1].kind == VARIABLE) {
 					if (equation[i-1].token.variable == IMAGINARY) {
 						d = fmod(equation[i+1].token.constant, 4.0);
 						if (d == 1.0) {
@@ -1761,8 +1752,7 @@ int		*np;
 							modified = true;
 						}
 					} else if ((equation[i-1].token.variable & VAR_MASK) == SIGN) {
-						d = fmod(equation[i+1].token.constant, 2.0);
-						equation[i+1].token.constant = d;
+						equation[i+1].token.constant = fmod(equation[i+1].token.constant, 2.0);
 					}
 				}
 			}
@@ -1795,20 +1785,16 @@ int		*np;
 		if (equation[i].token.operatr == DIVIDE) {
 			level = equation[i].level;
 			op = 0;
-			iloc = -1;
-			biloc = -1;
-			eiloc = -1;
+			iloc = biloc = eiloc = -1;
 			k = i;
 			for (j = i + 1; j < *np && equation[j].level > level; j++) {
-				if (equation[j].kind == OPERATOR
-				    && equation[j].level == level + 1) {
+				if (equation[j].kind == OPERATOR && equation[j].level == level + 1) {
 					op = equation[j].token.operatr;
 					k = j;
 					if (iloc >= 0 && eiloc < 0) {
 						eiloc = j;
 					}
-				} else if (equation[j].kind == VARIABLE
-				    && equation[j].token.variable == IMAGINARY) {
+				} else if (equation[j].kind == VARIABLE && equation[j].token.variable == IMAGINARY) {
 					if (iloc >= 0) {
 						op = 0;
 						break;
@@ -1828,8 +1814,7 @@ int		*np;
 				if (ilevel != level + 2) {
 					continue;
 				}
-				if (iloc > biloc
-				    && equation[iloc-1].token.operatr != TIMES)
+				if (iloc > biloc && equation[iloc-1].token.operatr != TIMES)
 					continue;
 				if (iloc + 1 < eiloc) {
 					switch (equation[iloc+1].token.operatr) {
@@ -1878,15 +1863,16 @@ int		*np;
 			scratch[j+(iloc-biloc)].token.constant = 1.0;
 			blt(&equation[iloc+2], &equation[iloc], (*np - iloc) * sizeof(token_type));
 			*np += 2;
-			equation[iloc].level = ilevel + 1;
+			ilevel++;
+			equation[iloc].level = ilevel;
 			equation[iloc].kind = CONSTANT;
 			equation[iloc].token.constant = -1.0;
 			iloc++;
-			equation[iloc].level = ilevel + 1;
+			equation[iloc].level = ilevel;
 			equation[iloc].kind = OPERATOR;
 			equation[iloc].token.operatr = TIMES;
 			iloc++;
-			equation[iloc].level = ilevel + 1;
+			equation[iloc].level = ilevel;
 			blt(&equation[i+1+n], &equation[i], (*np - i) * sizeof(token_type));
 			*np += n + 1;
 			blt(&equation[i+1], scratch, n * sizeof(token_type));
@@ -1913,6 +1899,32 @@ int		*np;
 	return(order_recurse(equation, np, 0, 1));
 }
 
+static inline void
+swap(equation, np, level, i1, i2)
+token_type	*equation;
+int		*np;
+int		level;
+int		i1, i2;
+{
+	int	e1, e2;
+	int	n1, n2;
+
+	for (e1 = i1 + 1;; e1 += 2) {
+		if (e1 >= *np || equation[e1].level <= level)
+			break;
+	}
+	for (e2 = i2 + 1;; e2 += 2) {
+		if (e2 >= *np || equation[e2].level <= level)
+			break;
+	}
+	n1 = e1 - i1;
+	n2 = e2 - i2;
+	blt(scratch, &equation[i1], (e2 - i1) * sizeof(token_type));
+	blt(&equation[i1+n2], &equation[e1], (i2 - e1) * sizeof(token_type));
+	blt(&equation[i1], &scratch[i2-i1], n2 * sizeof(token_type));
+	blt(&equation[e2-n1], scratch, n1 * sizeof(token_type));
+}
+
 static int
 order_recurse(equation, np, loc, level)
 token_type	*equation;
@@ -1994,9 +2006,9 @@ int		*np, loc, level;
 					if (equation[n].level <= level)
 						break;
 				n -= k;
-				blt(scratch, &equation[k], n * sizeof(*equation));
-				blt(&equation[j+n], &equation[j], (k - j) * sizeof(*equation));
-				blt(&equation[j], scratch, n * sizeof(*equation));
+				blt(scratch, &equation[k], n * sizeof(token_type));
+				blt(&equation[j+n], &equation[j], (k - j) * sizeof(token_type));
+				blt(&equation[j], scratch, n * sizeof(token_type));
 				j += n;
 				k += n;
 				modified = true;
@@ -2013,32 +2025,6 @@ corrupt:
 
 }
 
-static void
-swap(equation, np, level, i1, i2)
-token_type	*equation;
-int		*np;
-int		level;
-int		i1, i2;
-{
-	int	e1, e2;
-	int	n1, n2;
-
-	for (e1 = i1 + 1;; e1 += 2) {
-		if (e1 >= *np || equation[e1].level <= level)
-			break;
-	}
-	for (e2 = i2 + 1;; e2 += 2) {
-		if (e2 >= *np || equation[e2].level <= level)
-			break;
-	}
-	n1 = e1 - i1;
-	n2 = e2 - i2;
-	blt(scratch, &equation[i1], (e2 - i1) * sizeof(*equation));
-	blt(&equation[i1+n2], &equation[e1], (i2 - e1) * sizeof(*equation));
-	blt(&equation[i1], &scratch[i2-i1], n2 * sizeof(*equation));
-	blt(&equation[e2-n1], scratch, n1 * sizeof(*equation));
-}
-
 /*
  * Rationalize the denominator of algebraic fractions.
  * Only works with square roots.
@@ -2053,10 +2039,9 @@ int		*np;
 {
 	int	i, j, k;
 	int	i1, k1;
-	int	end_loc;
 	int	div_level;
+	int	end_loc, neg_one_loc = -1;
 	int	flag;
-	int	neg_one_loc = -1;
 	int	modified = false;
 	int	count;
 
@@ -2067,19 +2052,16 @@ be_thorough:
 		if (equation[i].token.operatr == DIVIDE) {
 			div_level = equation[i].level;
 			count = 0;
+			j = -1;
 			for (end_loc = i + 2; end_loc < *np && equation[end_loc].level > div_level; end_loc += 2) {
 				if (equation[end_loc].level == (div_level + 1)) {
 					count++;
+					if (j < 0) {
+						j = end_loc;
+					}
 				}
 			}
-			flag = false;
-			for (j = i + 2; j < end_loc; j += 2) {
-				if (equation[j].level == (div_level + 1)) {
-					flag = true;
-					break;
-				}
-			}
-			if (!flag)
+			if (j < 0)
 				continue;
 			switch (equation[j].token.operatr) {
 			case PLUS:
