@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator solve routines.
+ * Mathomatic solve routines.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -43,19 +43,20 @@ int	have;	/* equation number to solve */
 }
 
 /*
- * Main Mathomatic solve routine.
+ * Main Mathomatic symbolic solve routine.
  *
  * This works by moving everything containing the variable to solve for
  * to the LHS (via transposition), then moving everything not containing the variable to the
  * RHS.  Many tricks are used, and this routine works very well.
  *
- * "tlhs" and "trhs" are used by this routine.
+ * tlhs and trhs are used by this routine.
  *
  * Returns a positive integer if successful, with the result placed in the passed LHS and RHS.
  * Returns 1 for normal success.
  * Returns 2 if successful and a solution was zero and removed.
- * Returns 0 on failure.
+ * Returns 0 on failure.  Might succeed at a numeric solve.
  * Returns -1 if the equation is an identity.
+ * Returns -2 if unsolvable in all realms.
  */
 int
 solve_sub(wantp, wantn, leftp, leftnp, rightp, rightnp)
@@ -94,7 +95,7 @@ int		*rightnp;	/* size of RHS */
 		return false;
 	}
 	if (n_tlhs <= 0 || n_trhs <= 0) {
-		error(_("Please enter an equation."));
+		error(_("Please enter an equation or type \"help\"."));
 		return false;
 	}
 	if (wantp->kind == VARIABLE) {
@@ -170,11 +171,15 @@ no_simp:
 						}
 					}
 				}
-				if (op == TIMES || op == DIVIDE || op == POWER) {
+				switch (op) {
+				case TIMES:
+				case DIVIDE:
+				case POWER:
 					b1 = trhs;
 					op = PLUS;
 					for (p1 = b1; p1 < ep; p1++)
 						p1->level++;
+					break;
 				}
 				if (!g_of_f(op, b1, trhs, &n_trhs, tlhs, &n_tlhs))
 					return false;
@@ -210,7 +215,7 @@ zero_simp:
 					do {
 						simp_ssub(trhs, &n_trhs, 0L, 0.0, false, true, 0);
 					} while (uf_power(trhs, &n_trhs));
-				} while (super_factor(trhs, &n_trhs, true));
+				} while (super_factor(trhs, &n_trhs, 1));
 				list_tdebug(3);
 				ep = &trhs[n_trhs];
 				op = 0;
@@ -281,8 +286,7 @@ zero_simp:
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
 			if (!found) {
 				if ((p1 < ep || found_count || zsolve || n_tlhs > 1 || tlhs[0].kind != CONSTANT)
-				    && (p1 - b1 != 1 || b1->kind != CONSTANT
-				    || b1->token.constant != 1.0
+				    && (p1 - b1 != 1 || b1->kind != CONSTANT || b1->token.constant != 1.0
 				    || p1 >= ep || p1->token.operatr != DIVIDE)) {
 					if (op == 0) {
 						for (;; p1++) {
@@ -356,7 +360,7 @@ zero_simp:
 					if (se_compare(tlhs, n_tlhs, trhs, n_trhs, &diff_sign) && !diff_sign) {
 						error(_("This equation is an identity."));
 #if	!SILENT
-						printf(_("That is, the left equation side is identical to the right equation side.\n"));
+						printf(_("That is, the LHS is identical to the RHS.\n"));
 #endif
 						return -1;
 					}
@@ -380,7 +384,7 @@ zero_simp:
 					} else {
 						error(_("There are no possible values for the solve variable."));
 					}
-					return false;
+					return -2;
 				}
 				zflag = (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0);
 				if (need_flip >= found_count) {
@@ -507,7 +511,7 @@ zero_simp:
 					zero_solved = false;
 					qtries = 0;
 					if (!increase(b1->token.constant, v)) {
-						error(_("increase() function failed."));
+						error(_("Failed to raise both sides to a power."));
 						return false;
 					}
 					uf_flag = true;
@@ -592,8 +596,7 @@ lonely:
 					b1++;
 					break;
 				}
-				if (b1->kind == VARIABLE
-				    && b1->token.variable == v) {
+				if (b1->kind == VARIABLE && b1->token.variable == v) {
 					flag = true;
 				}
 				if (b1 == tlhs)
@@ -620,8 +623,7 @@ lonely:
 					b1++;
 					break;
 				}
-				if (b1->kind == VARIABLE
-				    && b1->token.variable == v) {
+				if (b1->kind == VARIABLE && b1->token.variable == v) {
 					flag = true;
 				}
 				if (b1 == tlhs)
@@ -639,8 +641,7 @@ lonely:
 	}
 	b1 = p1 + 1;
 	op = p1->token.operatr;
-	if (op == POWER && b1->level == 1
-	    && b1->kind == CONSTANT && b1->token.constant == d) {
+	if (op == POWER && b1->level == 1 && b1->kind == CONSTANT && b1->token.constant == d) {
 		return(g_of_f(POWER, b1, tlhs, &n_tlhs, trhs, &n_trhs));
 	}
 	if (!foundp) {
@@ -681,26 +682,24 @@ end:
  * where "x" is an expression containing passed variable "v",
  * and "n" is a constant.
  *
+ * The equation to solve is in tlhs and trhs, it must be solved for zero.
+ *
  * Return true if successful.
  */
 static int
 quad_solve(v)
-long	v;
+long	v;	/* solve variable */
 {
 	int		i, j, k;
 	token_type	*p1, *p2, *ep;
-	token_type	*x1p, *x2p;
-	token_type	*a1p, *a2p, *a2ep;
+	token_type	*x1p = NULL, *x2p;
+	token_type	*a1p = NULL, *a2p = NULL, *a2ep = NULL;
 	token_type	*b1p, *b2p, *b2ep;
 	token_type	*x1tp, *a1tp;
 	token_type	x1_storage[100];
-	int		nx1;
 	int		op, op2, opx1, opx2;
-	int		found;
-	int		diff_sign;
-	int		len;
-	int		alen, blen;
-	int		aloc;
+	int		found, diff_sign;
+	int		len, alen, blen, aloc, nx1;
 	double		high_power = 0.0;
 
 	uf_simp(trhs, &n_trhs);
@@ -1069,27 +1068,13 @@ int		*side2np;	/* pointer to the length of "side2p" */
 				op = PLUS;
 				continue;
 			case MODULUS:
-				if (op != MODULUS) {
-					printf(_("Something is wrong with the solve routine!\n"));
-				} else {
-					operandp = p1 + 1;
-				}
+				operandp = p1 + 1;
 				continue;
 			}
 			break;
 		}
 	}
 	operandn = p1 - operandp;
-	if (op == MODULUS) {
-		if (*side2np == 1 && side2p->kind == CONSTANT) {
-			if (side2p->token.constant < 0.0
-			    || (operandn == 1 && operandp->kind == CONSTANT
-			    && side2p->token.constant >= fabs(operandp->token.constant))) {
-				error(_("There are no solutions."));
-				return false;
-			}
-		}
-	}
 	if (op == POWER && operandp == side1p) {
 		if (!get_constant(side2p, *side2np, &d1))
 			return false;
@@ -1148,8 +1133,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		}
 	}
 #endif
-	if (*side1np + operandn + 3 > n_tokens
-	    || *side2np + operandn + 5 > n_tokens) {
+	if (*side1np + operandn + 3 > n_tokens || *side2np + operandn + 5 > n_tokens) {
 		error_huge();
 	}
 	for (p2 = side1p; p2 < ep; p2++)
@@ -1277,17 +1261,13 @@ int		*side2np;
 	blt(side2p + 2, side2p, *side2np * sizeof(*side2p));
 	*side2np += 2;
 
-	side1p->level = 1;
-	side1p->kind = CONSTANT;
-	side1p->token.constant = 1.0;
+	*side1p = one_token;
 	side1p++;
 	side1p->level = 1;
 	side1p->kind = OPERATOR;
 	side1p->token.operatr = DIVIDE;
 
-	side2p->level = 1;
-	side2p->kind = CONSTANT;
-	side2p->token.constant = 1.0;
+	*side2p = one_token;
 	side2p++;
 	side2p->level = 1;
 	side2p->kind = OPERATOR;
