@@ -1,21 +1,19 @@
 /*
- * Algebraic manipulator unfactorizing (expanding) routines.
+ * Mathomatic unfactorizing (expanding) routines.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
 
 static int	unf_sub();
-static int	usp_sub();
-static void	no_divide();
 
 /*
  * Totally unfactor equation side and simplify.
  */
 uf_simp(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to beginning of equation side */
+int		*np;		/* pointer to length of equation side */
 {
 	if (*np == 0)
 		return;
@@ -40,26 +38,36 @@ int		*np;
 /*
  * Unfactor times and divide only and simplify.
  */
+int
 uf_tsimp(equation, np)
 token_type	*equation;
 int		*np;
 {
-	uf_times(equation, np);
+	int	rv;
+
+	rv = uf_times(equation, np);
 	simp_loop(equation, np);
-	while (uf_times(equation, np))
+	if (uf_times(equation, np)) {
+		rv = true;
 		simp_loop(equation, np);
+	}
+	return rv;
 }
 
 /*
  * Totally unfactor equation side with no simplification.
  */
+int
 ufactor(equation, np)
 token_type	*equation;
 int		*np;
 {
+	int	rv;
+
 	uf_repeat(equation, np);
-	uf_times(equation, np);
+	rv = uf_times(equation, np);
 	uf_allpower(equation, np);
+	return rv;
 }
 
 /*
@@ -116,8 +124,10 @@ int		*np;
 }
 
 /*
- * Unfactor power only.
+ * Unfactor power only if it will help with expansion.
  * (a + 1)^2 -> (a + 1)*(a + 1)
+ *
+ * uf_times() is usually called after this.
  */
 uf_repeat(equation, np)
 token_type	*equation;
@@ -145,38 +155,7 @@ int		*np;
 	}
 }
 
-/*
- * Unfactor times and divide only.
- * (a + b)*c -> a*c + b*c
- * If "partial_flag", don't unfactor (a+b)/(c+d).
- *
- * Return true if equation side was modified.
- */
-int
-uf_times(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	i;
-	int	rv = false;
-
-	do {
-		organize(equation, np);
-		if (partial_flag) {
-			reorder(equation, np);
-		}
-		group_proc(equation, np);
-		if (partial_flag) {
-			no_divide(equation, np);
-		}
-		i = sub_ufactor(equation, np, 1);
-		rv |= i;
-	} while (i);
-	organize(equation, np);
-	return rv;
-}
-
-static void
+static inline void
 no_divide(equation, np)
 token_type	*equation;
 int		*np;
@@ -212,6 +191,37 @@ int		*np;
 	}
 }
 
+/*
+ * Unfactor times and divide only.
+ * (a + b)*c -> a*c + b*c
+ * If "partial_flag", don't unfactor (a+b)/(c+d).
+ *
+ * Return true if equation side was modified.
+ */
+int
+uf_times(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	i;
+	int	rv = false;
+
+	do {
+		organize(equation, np);
+		if (partial_flag) {
+			reorder(equation, np);
+		}
+		group_proc(equation, np);
+		if (partial_flag) {
+			no_divide(equation, np);
+		}
+		i = sub_ufactor(equation, np, 1);
+		rv |= i;
+	} while (i);
+	organize(equation, np);
+	return rv;
+}
+
 /*
  * General equation side algebraic expansion routine.
  * Expands everything.
@@ -305,14 +315,14 @@ u_again:
 				if (len + (eb1 - b1) + (i - b2) + (e1 - be1) + 1 > n_tokens) {
 					error_huge();
 				}
-				blt(&scratch[len], &equation[b1], (eb1 - b1) * sizeof(*equation));
+				blt(&scratch[len], &equation[b1], (eb1 - b1) * sizeof(token_type));
 				j = len;
 				len += (eb1 - b1);
 				for (; j < len; j++)
 					scratch[j].level++;
-				blt(&scratch[len], &equation[b2], (i - b2) * sizeof(*equation));
+				blt(&scratch[len], &equation[b2], (i - b2) * sizeof(token_type));
 				len += (i - b2);
-				blt(&scratch[len], &equation[be1], (e1 - be1) * sizeof(*equation));
+				blt(&scratch[len], &equation[be1], (e1 - be1) * sizeof(token_type));
 				j = len;
 				len += (e1 - be1);
 				for (; j < len; j++)
@@ -331,9 +341,9 @@ u_again:
 					if (*np - (e1 - b1) + len > n_tokens) {
 						error_huge();
 					}
-					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(*equation));
+					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(token_type));
 					*np += len - (e1 - b1);
-					blt(&equation[b1], scratch, len * sizeof(*equation));
+					blt(&equation[b1], scratch, len * sizeof(token_type));
 					return true;
 				}
 			}
@@ -357,9 +367,9 @@ u1_again:
 				if (len + (i - b2) + (e1 - loc) + 1 > n_tokens) {
 					error_huge();
 				}
-				blt(&scratch[len], &equation[b2], (i - b2) * sizeof(*equation));
+				blt(&scratch[len], &equation[b2], (i - b2) * sizeof(token_type));
 				len += (i - b2);
-				blt(&scratch[len], &equation[loc], (e1 - loc) * sizeof(*equation));
+				blt(&scratch[len], &equation[loc], (e1 - loc) * sizeof(token_type));
 				j = len;
 				len += (e1 - loc);
 				for (; j < len; j++)
@@ -378,9 +388,9 @@ u1_again:
 					if (*np - (e1 - b1) + len > n_tokens) {
 						error_huge();
 					}
-					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(*equation));
+					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(token_type));
 					*np += len - (e1 - b1);
-					blt(&equation[b1], scratch, len * sizeof(*equation));
+					blt(&equation[b1], scratch, len * sizeof(token_type));
 					return true;
 				}
 			}
@@ -404,11 +414,11 @@ u2_again:
 					error_huge();
 				}
 				j = len;
-				blt(&scratch[len], &equation[b1], (loc + 1 - b1) * sizeof(*equation));
+				blt(&scratch[len], &equation[b1], (loc + 1 - b1) * sizeof(token_type));
 				len += (loc + 1 - b1);
 				for (; j < len; j++)
 					scratch[j].level++;
-				blt(&scratch[len], &equation[b2], (i - b2) * sizeof(*equation));
+				blt(&scratch[len], &equation[b2], (i - b2) * sizeof(token_type));
 				len += (i - b2);
 				if (i < e1) {
 					scratch[len].level = level;
@@ -428,9 +438,9 @@ u2_again:
 					if (*np - (e1 - b1) + len > n_tokens) {
 						error_huge();
 					}
-					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(*equation));
+					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(token_type));
 					*np += len - (e1 - b1);
-					blt(&equation[b1], scratch, len * sizeof(*equation));
+					blt(&equation[b1], scratch, len * sizeof(token_type));
 					return true;
 				}
 			}
@@ -480,32 +490,7 @@ do_repeat:
 	return false;
 }
 
-/*
- * Convert a/(x^y) to a*x^(-1*y).
- * If y is a constant, don't do.
- *
- * Return true if equation side is modified.
- */
-int
-unsimp_power(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	modified = false;
-	int	i;
-
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			if (equation[i].level == equation[i+1].level
-			    && equation[i+1].kind == CONSTANT)
-				continue;
-			modified |= usp_sub(equation, np, i);
-		}
-	}
-	return modified;
-}
-
-static int
+static inline int
 usp_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -534,7 +519,7 @@ int		*np, i;
 		equation[j].level++;
 	}
 	i++;
-	blt(&equation[i+2], &equation[i], (*np - i) * sizeof(*equation));
+	blt(&equation[i+2], &equation[i], (*np - i) * sizeof(token_type));
 	*np += 2;
 	equation[i].level = level + 1;
 	equation[i].kind = CONSTANT;
@@ -546,6 +531,31 @@ int		*np, i;
 	return true;
 }
 
+/*
+ * Convert a/(x^y) to a*x^(-1*y).
+ * If y is a constant, don't do.
+ *
+ * Return true if equation side is modified.
+ */
+int
+unsimp_power(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	modified = false;
+	int	i;
+
+	for (i = 1; i < *np; i += 2) {
+		if (equation[i].token.operatr == POWER) {
+			if (equation[i].level == equation[i+1].level
+			    && equation[i+1].kind == CONSTANT)
+				continue;
+			modified |= usp_sub(equation, np, i);
+		}
+	}
+	return modified;
+}
+
 #if	false
 /*
  * Convert a/(x^y) to a*(1/x)^y.
@@ -597,7 +607,7 @@ int		*np, i;
 	for (k = j; k < i; k++) {
 		equation[k].level++;
 	}
-	blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
+	blt(&equation[j+2], &equation[j], (*np - j) * sizeof(token_type));
 	*np += 2;
 	equation[j].level = level + 1;
 	equation[j].kind = CONSTANT;
@@ -631,7 +641,7 @@ int		*np;
 					if ((*np + 2) > n_tokens) {
 						error_huge();
 					}
-					blt(&equation[i+3], &equation[i+1], (*np - (i + 1)) * sizeof(*equation));
+					blt(&equation[i+3], &equation[i+1], (*np - (i + 1)) * sizeof(token_type));
 					*np += 2;
 					equation[i].token.constant = -equation[i].token.constant;
 					i++;
@@ -652,13 +662,17 @@ int		*np;
 /*
  * Convert 1/(k1^k2) to 1/(k1^(k2-1))/k1 if k1 is integer,
  * otherwise 1*((1/k1)^k2).
+ *
+ * Return true if equation side was modified.
  */
+int
 patch_root_div(equation, np)
 token_type	*equation;
 int		*np;
 {
 	int	i;
 	int	level;
+	int	modified = false;
 
 	for (i = 1; i < *np - 2; i += 2) {
 		if (equation[i].token.operatr == DIVIDE) {
@@ -687,7 +701,9 @@ int		*np;
 					equation[i].token.operatr = TIMES;
 					equation[i+1].token.constant = 1.0 / equation[i+1].token.constant;
 				}
+				modified = true;
 			}
 		}
 	}
+	return modified;
 }
